@@ -62,7 +62,6 @@ DefaultBtAnnounce::DefaultBtAnnounce(BtContextHandle btContext,
 {
   prevAnnounceTime.setTimeInSec(0);
   key = generateKey();
-  peerId = generatePeerId();
   logger = LogFactory::getInstance();
 }
 
@@ -73,12 +72,6 @@ string DefaultBtAnnounce::generateKey() const {
   return Util::randomAlpha(8);
 }
 
-string DefaultBtAnnounce::generatePeerId() const {
-  string peerId = "-aria2-";
-  peerId += Util::randomAlpha(20-peerId.size());
-  return peerId;
-}
-
 bool DefaultBtAnnounce::isDefaultAnnounceReady() {
   return (trackers == 0 && prevAnnounceTime.elapsed(minInterval));
 }
@@ -91,7 +84,7 @@ bool DefaultBtAnnounce::isStoppedAnnounceReady() {
 
 bool DefaultBtAnnounce::isCompletedAnnounceReady() {
   return (trackers == 0 &&
-	  pieceStorage->downloadFinished() &&
+	  pieceStorage->allDownloadFinished() &&
 	  announceList.countCompletedAllowedTier());
 }
 
@@ -113,7 +106,7 @@ string DefaultBtAnnounce::getAnnounceUrl() {
     // If download completed before "started" event is sent to a tracker,
     // we change the event to something else to prevent us from
     // sending "completed" event.
-    if(pieceStorage->downloadFinished() &&
+    if(pieceStorage->allDownloadFinished() &&
        announceList.getEvent() == AnnounceTier::STARTED) {
       announceList.setEvent(AnnounceTier::STARTED_AFTER_COMPLETION);
     }
@@ -131,7 +124,7 @@ string DefaultBtAnnounce::getAnnounceUrl() {
   string url = announceList.getAnnounce()+"?"+
     "info_hash="+Util::torrentUrlencode(btContext->getInfoHash(),
 					btContext->getInfoHashLength())+"&"+
-    "peer_id="+peerId+"&"+
+    "peer_id="+Util::torrentUrlencode(btContext->getPeerId(), 20)+"&"+
     "port="+Util::itos(btRuntime->getListenPort())+"&"+
     "uploaded="+Util::llitos(stat.getSessionUploadLength())+"&"+
     "downloaded="+Util::llitos(stat.getSessionDownloadLength())+"&"+
