@@ -41,7 +41,7 @@
 #include "Logger.h"
 #include "BtRuntime.h"
 
-#define MAX_PEER_LIST_SIZE 100
+#define MAX_PEER_LIST_SIZE 60
 #define MAX_PEER_ERROR 5
 
 class DefaultPeerStorage : public PeerStorage {
@@ -49,10 +49,12 @@ private:
   BtContextHandle btContext;
   const Option* option;
   Peers peers;
+  Peers incomingPeers;
   int maxPeerListSize;
-  int peerEntryIdCounter;
   Logger* logger;
   BtRuntimeHandle btRuntime;
+  long long int removedPeerSessionDownloadLength;
+  long long int removedPeerSessionUploadLength;
 public:
   DefaultPeerStorage(BtContextHandle btContext, const Option* option);
   virtual ~DefaultPeerStorage();
@@ -64,7 +66,9 @@ public:
 
   virtual bool addPeer(const PeerHandle& peer);
 
-  int countPeer() const;
+  virtual bool addIncomingPeer(const PeerHandle& peer);
+
+  int32_t countPeer() const;
 
   virtual PeerHandle getUnusedPeer();
 
@@ -80,11 +84,17 @@ public:
 
   virtual TransferStat calculateStat();
 
+  virtual void returnPeer(const PeerHandle& peer);
+
   void setMaxPeerListSize(int size) { this->maxPeerListSize = size; }
  
   int getMaxPeerListSize() const { return maxPeerListSize; }
 
   void deleteUnusedPeer(int delSize);
+  
+  void onErasingPeer(const PeerHandle& peer);
+
+  const Peers& getIncomingPeers() const { return incomingPeers; }
 };
 
 #endif // _D_DEFAULT_PEER_STORAGE_H_
