@@ -39,9 +39,11 @@
 #include "MultiDiskAdaptor.h"
 #include "CopyDiskAdaptor.h"
 #include "DefaultDiskWriter.h"
-#include "MultiDiskWriter.h"
-#include "PreAllocationDiskWriter.h"
 #include "DlAbortEx.h"
+#include "BitfieldManFactory.h"
+#include "FileAllocationMonitor.h"
+#include "DiskAdaptorWriter.h"
+#include "ChunkChecksumValidator.h"
 
 DefaultPieceStorage::DefaultPieceStorage(BtContextHandle btContext, const Option* option):
   btContext(btContext),
@@ -49,8 +51,10 @@ DefaultPieceStorage::DefaultPieceStorage(BtContextHandle btContext, const Option
   endGamePieceNum(END_GAME_PIECE_NUM),
   option(option)
 {
-  bitfieldMan = new BitfieldMan(btContext->getPieceLength(),
-				btContext->getTotalLength());
+  bitfieldMan =
+    BitfieldManFactory::getFactoryInstance()->
+    createBitfieldMan(btContext->getPieceLength(),
+		      btContext->getTotalLength());
   logger = LogFactory::getInstance();
 }
 
@@ -79,15 +83,15 @@ int DefaultPieceStorage::getMissingPieceIndex(const PeerHandle& peer) {
   return index;
 }
 
-Piece DefaultPieceStorage::checkOutPiece(int index) {
+PieceHandle DefaultPieceStorage::checkOutPiece(int index) {
   if(index == -1) {
-    return Piece::nullPiece;
+    return 0;
   }
   bitfieldMan->setUseBit(index);
 
-  Piece piece = findUsedPiece(index);
-  if(Piece::isNull(piece)) {
-    Piece piece(index, bitfieldMan->getBlockLength(index));
+  PieceHandle piece = findUsedPiece(index);
+  if(piece.isNull()) {
+    piece = new Piece(index, bitfieldMan->getBlockLength(index));
     addUsedPiece(piece);
     return piece;
   } else {
@@ -95,8 +99,26 @@ Piece DefaultPieceStorage::checkOutPiece(int index) {
   }
 }
 
-void DefaultPieceStorage::addUsedPiece(const Piece& piece) {
-  // TODO ? if nullPiece
+/**
+ * Newly instantiated piece is not added to usedPieces.
+ * Because it is waste of memory and there is no chance to use them later.
+ */
+PieceHandle DefaultPieceStorage::getPiece(int index) {
+  if(0 <= index && index <= bitfieldMan->getMaxIndex()) {
+    PieceHandle piece = findUsedPiece(index);
+    if(piece.isNull()) {
+      piece = new Piece(index, bitfieldMan->getBlockLength(index));
+      if(hasPiece(index)) {
+	piece->setAllBlock();
+      }
+    }
+    return piece;
+  } else {
+    return 0;
+  }
+}
+
+void DefaultPieceStorage::addUsedPiece(const PieceHandle& piece) {
   usedPieces.push_back(piece);
 }
 
@@ -106,23 +128,23 @@ private:
 public:
   FindPiece(int index):index(index) {}
 
-  bool operator()(const Piece& piece) {
-    return piece.getIndex() == index;
+  bool operator()(const PieceHandle& piece) {
+    return piece->getIndex() == index;
   }
 };
 
-Piece DefaultPieceStorage::findUsedPiece(int index) const {
+PieceHandle DefaultPieceStorage::findUsedPiece(int index) const {
   Pieces::const_iterator itr = find_if(usedPieces.begin(),
 				       usedPieces.end(),
 				       FindPiece(index));
   if(itr == usedPieces.end()) {
-    return Piece::nullPiece;
+    return 0;
   } else {
     return *itr;
   }
 }
 
-Piece DefaultPieceStorage::getMissingPiece(const PeerHandle& peer) {
+PieceHandle DefaultPieceStorage::getMissingPiece(const PeerHandle& peer) {
   int index = getMissingPieceIndex(peer);
   return checkOutPiece(index);
 }
@@ -149,13 +171,13 @@ int DefaultPieceStorage::getMissingFastPieceIndex(const PeerHandle& peer) {
   return index;
 }
 
-Piece DefaultPieceStorage::getMissingFastPiece(const PeerHandle& peer) {
+PieceHandle DefaultPieceStorage::getMissingFastPiece(const PeerHandle& peer) {
   int index = getMissingFastPieceIndex(peer);
   return checkOutPiece(index);
 }
 
-void DefaultPieceStorage::deleteUsedPiece(const Piece& piece) {
-  if(Piece::isNull(piece)) {
+void DefaultPieceStorage::deleteUsedPiece(const PieceHandle& piece) {
+  if(piece.isNull()) {
     return;
   }
   Pieces::iterator itr = find(usedPieces.begin(), usedPieces.end(), piece);
@@ -185,12 +207,12 @@ int DefaultPieceStorage::deleteUsedPiecesByFillRate(int fillRate,
   int deleted = 0;
   for(Pieces::iterator itr = usedPieces.begin();
       itr != usedPieces.end() && deleted < toDelete;) {
-    Piece& piece = *itr;
-    if(!bitfieldMan->isUseBitSet(piece.getIndex()) &&
-       piece.countCompleteBlock() <= piece.countBlock()*(fillRate/100.0)) {
+    PieceHandle& piece = *itr;
+    if(!bitfieldMan->isUseBitSet(piece->getIndex()) &&
+       piece->countCompleteBlock() <= piece->countBlock()*(fillRate/100.0)) {
       logger->debug("Deleting used piece index=%d, fillRate(%%)=%d<=%d",
-		    piece.getIndex(),
-		    (piece.countCompleteBlock()*100)/piece.countBlock(),
+		    piece->getIndex(),
+		    (piece->countCompleteBlock()*100)/piece->countBlock(),
 		    fillRate);
       itr = usedPieces.erase(itr);
       deleted++;
@@ -201,24 +223,26 @@ int DefaultPieceStorage::deleteUsedPiecesByFillRate(int fillRate,
   return deleted;
 }
 
-void DefaultPieceStorage::completePiece(const Piece& piece) {
-  if(Piece::isNull(piece)) {
+void DefaultPieceStorage::completePiece(const PieceHandle& piece) {
+  if(piece.isNull()) {
     return;
   }
   deleteUsedPiece(piece);
   if(!isEndGame()) {
     reduceUsedPieces(100);
   }
-  if(downloadFinished()) {
+  if(allDownloadFinished()) {
     return;
   }
-  bitfieldMan->setBit(piece.getIndex());
-  bitfieldMan->unsetUseBit(piece.getIndex());
+  bitfieldMan->setBit(piece->getIndex());
+  bitfieldMan->unsetUseBit(piece->getIndex());
   if(downloadFinished()) {
     diskAdaptor->onDownloadComplete();
     if(isSelectiveDownloadingMode()) {
       logger->notice(_("Download of selected files was complete."));
-      finishSelectiveDownloadingMode();
+      // following line was commented out in order to stop sending request
+      // message after user-specified files were downloaded.
+      //finishSelectiveDownloadingMode();
     } else {
       logger->info(_("The download was complete."));
     }
@@ -235,50 +259,18 @@ void DefaultPieceStorage::finishSelectiveDownloadingMode() {
 }
 
 // not unittested
-void DefaultPieceStorage::cancelPiece(const Piece& piece) {
-  if(Piece::isNull(piece)) {
+void DefaultPieceStorage::cancelPiece(const PieceHandle& piece) {
+  if(piece.isNull()) {
     return;
   }
-  updatePiece(piece);
-  bitfieldMan->unsetUseBit(piece.getIndex());
+  bitfieldMan->unsetUseBit(piece->getIndex());
   if(!isEndGame()) {
-    if(piece.countCompleteBlock() == 0) {
+    if(piece->countCompleteBlock() == 0) {
       deleteUsedPiece(piece);
     }
   }
 }
 
-// not unittested
-void DefaultPieceStorage::updatePiece(const Piece& piece) {
-  if(Piece::isNull(piece)) {
-    return;
-  }
-  Pieces::iterator itr = find(usedPieces.begin(), usedPieces.end(),
-			      piece);
-  if(itr != usedPieces.end()) {
-    *itr = piece;
-  }
-}
-
-// not unittested
-void DefaultPieceStorage::syncPiece(Piece& piece) {
-  if(Piece::isNull(piece)) {
-    return;
-  }
-  Pieces::iterator itr = find(usedPieces.begin(), usedPieces.end(),
-			      piece);
-  if(itr != usedPieces.end()) {
-    piece = *itr;
-    return;
-  } else {
-    // hasPiece(piece.getIndex()) is true, then set all bit of
-    // piece.bitfield to 1
-    if(hasPiece(piece.getIndex())) {
-      piece.setAllBlock();
-    }
-  }
-}
-
 bool DefaultPieceStorage::hasPiece(int index) {
   return bitfieldMan->isBitSet(index);
 }
@@ -321,8 +313,8 @@ void DefaultPieceStorage::setFileFilter(const Integers& fileIndexes) {
   const FileEntries& entries = diskAdaptor->getFileEntries();
   for(int i = 0; i < (int)entries.size(); i++) {
     if(find(fileIndexes.begin(), fileIndexes.end(), i+1) != fileIndexes.end()) {
-      logger->debug("index=%d is %s", i+1, entries.at(i)->getPath().c_str());
-      filePaths.push_back(entries.at(i)->getPath());
+      logger->debug("index=%d is %s", i+1, entries[i]->getPath().c_str());
+      filePaths.push_back(entries[i]->getPath());
     }
   }
   setFileFilter(filePaths);
@@ -336,28 +328,40 @@ void DefaultPieceStorage::clearFileFilter() {
 
 // not unittested
 bool DefaultPieceStorage::downloadFinished() {
+  return bitfieldMan->isFilteredAllBitSet();
+}
+
+// not unittested
+bool DefaultPieceStorage::allDownloadFinished() {
   return bitfieldMan->isAllBitSet();
 }
 
 // not unittested
 void DefaultPieceStorage::initStorage() {
-  if(diskAdaptor) {
-    delete diskAdaptor;
-    diskAdaptor = 0;
-  }
   if(option->get(PREF_DIRECT_FILE_MAPPING) == V_TRUE) {
     if(btContext->getFileMode() == BtContext::SINGLE) {
-      diskAdaptor = new DirectDiskAdaptor(new DefaultDiskWriter(btContext->getTotalLength()));
+      DefaultDiskWriterHandle writer = DefaultDiskWriter::createNewDiskWriter(option);
+      DirectDiskAdaptorHandle directDiskAdaptor = new DirectDiskAdaptor();
+      directDiskAdaptor->setDiskWriter(writer);
+      directDiskAdaptor->setTotalLength(btContext->getTotalLength());
+      this->diskAdaptor = directDiskAdaptor;
     } else {
-      diskAdaptor = new MultiDiskAdaptor(new MultiDiskWriter(btContext->getPieceLength()));
-      ((MultiDiskAdaptor*)diskAdaptor)->setTopDir(btContext->getName());
+      MultiDiskAdaptorHandle multiDiskAdaptor = new MultiDiskAdaptor();
+      multiDiskAdaptor->setPieceLength(btContext->getPieceLength());
+      multiDiskAdaptor->setTopDir(btContext->getName());
+      multiDiskAdaptor->setOption(option);
+      this->diskAdaptor = multiDiskAdaptor;
     }
   } else {
-    diskAdaptor = new CopyDiskAdaptor(new PreAllocationDiskWriter(btContext->getTotalLength()));
-    ((CopyDiskAdaptor*)diskAdaptor)->setTempFilename(btContext->getName()+".a2tmp");
+    DefaultDiskWriterHandle writer = DefaultDiskWriter::createNewDiskWriter(option);
+    CopyDiskAdaptorHandle copyDiskAdaptor = new CopyDiskAdaptor();
+    copyDiskAdaptor->setDiskWriter(writer);
+    copyDiskAdaptor->setTempFilename(btContext->getName()+".a2tmp");
+    copyDiskAdaptor->setTotalLength(btContext->getTotalLength());
     if(btContext->getFileMode() == BtContext::MULTI) {
-      ((CopyDiskAdaptor*)diskAdaptor)->setTopDir(btContext->getName());
+      copyDiskAdaptor->setTopDir(btContext->getName());
     }
+    this->diskAdaptor = copyDiskAdaptor;
   }
   string storeDir = option->get(PREF_DIR);
   if(storeDir == "") {
@@ -380,7 +384,7 @@ const unsigned char* DefaultPieceStorage::getBitfield() {
   return bitfieldMan->getBitfield();
 }
 
-DiskAdaptor* DefaultPieceStorage::getDiskAdaptor() {
+DiskAdaptorHandle DefaultPieceStorage::getDiskAdaptor() {
   return diskAdaptor;
 }
 
@@ -433,3 +437,20 @@ void DefaultPieceStorage::removeAdvertisedPiece(int elapsed) {
     haves.erase(itr, haves.end());
   }
 }
+
+void DefaultPieceStorage::markAllPiecesDone()
+{
+  bitfieldMan->setAllBit();
+}
+
+void DefaultPieceStorage::checkIntegrity()
+{
+  logger->notice("Validating file %s",
+		 diskAdaptor->getFilePath().c_str());
+  ChunkChecksumValidator v;
+  v.setDigestAlgo(DIGEST_ALGO_SHA1);
+  v.setDiskWriter(new DiskAdaptorWriter(diskAdaptor));
+  v.setFileAllocationMonitor(FileAllocationMonitorFactory::getFactory()->createNewMonitor());
+  v.validate(bitfieldMan, btContext->getPieceHashes(),
+	     btContext->getPieceLength());
+}
