@@ -42,6 +42,7 @@
 #include "Logger.h"
 #include "Option.h"
 #include "Piece.h"
+#include "FileAllocator.h"
 
 #define END_GAME_PIECE_NUM 20
 
@@ -68,38 +69,36 @@ class DefaultPieceStorage : public PieceStorage {
 private:
   BtContextHandle btContext;
   BitfieldMan* bitfieldMan;
-  DiskAdaptor* diskAdaptor;
+  DiskAdaptorHandle diskAdaptor;
   Pieces usedPieces;
-  int endGamePieceNum;
+  int32_t endGamePieceNum;
   Logger* logger;
   const Option* option;
   Haves haves;
+  FileAllocatorHandle createFileAllocator();
 
   int getMissingPieceIndex(const PeerHandle& peer);
   int getMissingFastPieceIndex(const PeerHandle& peer);
-  Piece checkOutPiece(int index);
-  void addUsedPiece(const Piece& piece);
-  Piece findUsedPiece(int index) const;
+  PieceHandle checkOutPiece(int index);
   int deleteUsedPiecesByFillRate(int fillRate, int toDelete);
   void reduceUsedPieces(int delMax);
-  void deleteUsedPiece(const Piece& piece);
+  void deleteUsedPiece(const PieceHandle& piece);
+  PieceHandle findUsedPiece(int index) const;
 public:
   DefaultPieceStorage(BtContextHandle btContext, const Option* option);
   virtual ~DefaultPieceStorage();
 
   virtual bool hasMissingPiece(const PeerHandle& peer);
 
-  virtual Piece getMissingPiece(const PeerHandle& peer);
+  virtual PieceHandle getMissingPiece(const PeerHandle& peer);
 
-  virtual Piece getMissingFastPiece(const PeerHandle& peer);
+  virtual PieceHandle getMissingFastPiece(const PeerHandle& peer);
 
-  virtual void completePiece(const Piece& piece);
+  virtual PieceHandle getPiece(int index);
 
-  virtual void cancelPiece(const Piece& piece);
+  virtual void completePiece(const PieceHandle& piece);
 
-  virtual void updatePiece(const Piece& piece);
-
-  virtual void syncPiece(Piece& piece);
+  virtual void cancelPiece(const PieceHandle& piece);
 
   virtual bool hasPiece(int index);
 
@@ -121,6 +120,8 @@ public:
 
   virtual bool downloadFinished();
 
+  virtual bool allDownloadFinished();
+
   virtual void setBitfield(const unsigned char* bitfield,
 			   int bitfieldLength);
   
@@ -128,11 +129,11 @@ public:
 
   virtual const unsigned char* getBitfield();
 
-  void setEndGamePieceNum(int num) {
+  void setEndGamePieceNum(int32_t num) {
     endGamePieceNum = num;
   }
 
-  int getEndGamePieceNum() const {
+  int32_t getEndGamePieceNum() const {
     return endGamePieceNum;
   }
 
@@ -142,7 +143,7 @@ public:
 
   virtual bool isEndGame();
   
-  virtual DiskAdaptor* getDiskAdaptor();
+  virtual DiskAdaptorHandle getDiskAdaptor();
 
   virtual int getPieceLength(int index);
 
@@ -153,6 +154,15 @@ public:
 
   virtual void removeAdvertisedPiece(int elapsed);
 
+  virtual void markAllPiecesDone();
+
+  virtual void checkIntegrity();
+
+  /**
+   * This method is made private for test purpose only.
+   */
+  void addUsedPiece(const PieceHandle& piece);
+
 };
 
 #endif // _D_DEFAULT_PIECE_STORAGE_H_
