@@ -36,41 +36,9 @@
 #include "DlAbortEx.h"
 #include "LogFactory.h"
 
-DiskAdaptor::DiskAdaptor(DiskWriter* diskWriter):diskWriter(diskWriter) {
-  logger = LogFactory::getInstance();
-}
+DiskAdaptor::DiskAdaptor():logger(LogFactory::getInstance()) {}
 
-DiskAdaptor::~DiskAdaptor() {
-  delete diskWriter;
-}
-
-void DiskAdaptor::openFile() {
-  diskWriter->openFile(getFilePath());
-}
-
-void DiskAdaptor::closeFile() {
-  diskWriter->closeFile();
-}
-
-void DiskAdaptor::openExistingFile() {
-  diskWriter->openExistingFile(getFilePath());
-}
-
-void DiskAdaptor::initAndOpenFile() {
-  diskWriter->initAndOpenFile(getFilePath());
-}
-
-void DiskAdaptor::writeData(const char* data, int len, long long int position) {
-  diskWriter->writeData(data, len, position);
-}
-
-int DiskAdaptor::readData(char* data, int len, long long int position) {
-  return diskWriter->readData(data, len, position);
-}
-
-string DiskAdaptor::sha1Sum(long long int offset, long long int length) {
-  return diskWriter->sha1Sum(offset, length);
-}
+DiskAdaptor::~DiskAdaptor() {}
 
 FileEntryHandle DiskAdaptor::getFileEntryFromPath(const string& fileEntryPath) const {
   for(FileEntries::const_iterator itr = fileEntries.begin();
@@ -79,7 +47,7 @@ FileEntryHandle DiskAdaptor::getFileEntryFromPath(const string& fileEntryPath) c
       return *itr;
     }
   }
-  throw new DlAbortEx("no such file entry <%s>", fileEntryPath.c_str());
+  throw new DlAbortEx("No such file entry <%s>", fileEntryPath.c_str());
 }
 
 bool DiskAdaptor::addDownloadEntry(const string& fileEntryPath) {
@@ -97,7 +65,7 @@ bool DiskAdaptor::addDownloadEntry(int index) {
   if(fileEntries.size() <= (unsigned int)index) {
     return false;
   }
-  fileEntries.at(index)->setRequested(true);
+  fileEntries[index]->setRequested(true);
   return true;
 }
 
