@@ -36,8 +36,14 @@
 #include "prefs.h"
 #include "DefaultDiskWriter.h"
 #include "InitiateConnectionCommandFactory.h"
-#include "ByteArrayDiskWriter.h"
 #include "Util.h"
+#include "FileAllocator.h"
+#include "FileAllocationMonitor.h"
+#include "FillRequestGroupCommand.h"
+#include "CUIDCounter.h"
+#include "FileAllocationDispatcherCommand.h"
+#include "FileAllocationMan.h"
+#include "CheckIntegrityMan.h"
 #ifdef ENABLE_BITTORRENT
 # include "PeerListenCommand.h"
 # include "TrackerWatcherCommand.h"
@@ -46,6 +52,7 @@
 # include "SeedCheckCommand.h"
 # include "PeerChokeCommand.h"
 # include "HaveEraseCommand.h"
+# include "ActivePeerConnectionCommand.h"
 # include "UnionSeedCriteria.h"
 # include "TimeSeedCriteria.h"
 # include "ShareRatioSeedCriteria.h"
@@ -55,6 +62,38 @@
 # include "DefaultBtProgressInfoFile.h"
 #endif // ENABLE_BITTORRENT
 
+ConsoleDownloadEngine*
+DownloadEngineFactory::newConsoleEngine(const Option* op,
+					const RequestGroups& requestGroups)
+{
+  // set PREF_OUT parameter to requestGroup in non-multi download mode.
+  if(requestGroups.size() == 1) {
+    requestGroups.front()->setUserDefinedFilename(op->get(PREF_OUT));
+  }
+  RequestGroups workingSet;
+  RequestGroups reservedSet;
+  if(op->getAsInt(PREF_MAX_CONCURRENT_DOWNLOADS) < (int32_t)requestGroups.size()) {
+    copy(requestGroups.begin(), requestGroups.begin()+op->getAsInt(PREF_MAX_CONCURRENT_DOWNLOADS), back_inserter(workingSet));
+    copy(requestGroups.begin()+op->getAsInt(PREF_MAX_CONCURRENT_DOWNLOADS),
+	 requestGroups.end(), back_inserter(reservedSet));
+  } else {
+    workingSet = requestGroups;
+  }
+
+  ConsoleDownloadEngine* e = new ConsoleDownloadEngine();
+  e->option = op;
+  RequestGroupManHandle requestGroupMan = new RequestGroupMan(workingSet,
+							      op->getAsInt(PREF_MAX_CONCURRENT_DOWNLOADS));
+  requestGroupMan->addReservedGroup(reservedSet);
+  e->_requestGroupMan = requestGroupMan;
+  e->_fileAllocationMan = new FileAllocationMan();
+  e->_checkIntegrityMan = new CheckIntegrityMan();
+
+  e->commands.push_back(new FillRequestGroupCommand(CUIDCounterSingletonHolder::instance()->newID(), e, 1));
+  e->commands.push_back(new FileAllocationDispatcherCommand(CUIDCounterSingletonHolder::instance()->newID(), e));
+  return e;
+}
+
 ConsoleDownloadEngine*
 DownloadEngineFactory::newConsoleEngine(const Option* op,
 					const Requests& requests,
@@ -62,19 +101,19 @@ DownloadEngineFactory::newConsoleEngine(const Option* op,
 {
   ConsoleDownloadEngine* e = new ConsoleDownloadEngine();
   e->option = op;
-  e->segmentMan = new SegmentMan();
-  e->segmentMan->diskWriter = new DefaultDiskWriter();
-  e->segmentMan->dir = op->get(PREF_DIR);
-  e->segmentMan->ufilename = op->get(PREF_OUT);
-  e->segmentMan->option = op;
-  e->segmentMan->reserved = reserved;
+//   e->segmentMan = new SegmentMan();
+//   e->segmentMan->diskWriter = DefaultDiskWriter::createNewDiskWriter(op);
+//   e->segmentMan->dir = op->get(PREF_DIR);
+//   e->segmentMan->ufilename = op->get(PREF_OUT);
+//   e->segmentMan->option = op;
+//   e->segmentMan->reserved = reserved;
   
-  int cuidCounter = 1;
-  for(Requests::const_iterator itr = requests.begin();
-      itr != requests.end();
-      itr++, cuidCounter++) {
-    e->commands.push_back(InitiateConnectionCommandFactory::createInitiateConnectionCommand(cuidCounter, *itr, e));
-  }
+//   int cuidCounter = 1;
+//   for(Requests::const_iterator itr = requests.begin();
+//       itr != requests.end();
+//       itr++, cuidCounter++) {
+//     e->commands.push_back(InitiateConnectionCommandFactory::createInitiateConnectionCommand(cuidCounter, *itr, e));
+//   }
   return e;
 }
 
@@ -86,10 +125,12 @@ DownloadEngineFactory::newTorrentConsoleEngine(const BtContextHandle& btContext,
 {
   TorrentConsoleDownloadEngine* te = new TorrentConsoleDownloadEngine();
   te->option = op;
-  ByteArrayDiskWriter* byteArrayDiskWriter = new ByteArrayDiskWriter();
-  te->segmentMan = new SegmentMan();
-  te->segmentMan->diskWriter = byteArrayDiskWriter;
-  te->segmentMan->option = op;
+  RequestGroupManHandle requestGroupMan = new RequestGroupMan();
+  te->_requestGroupMan = requestGroupMan;
+  //  ByteArrayDiskWriter* byteArrayDiskWriter = new ByteArrayDiskWriter();
+//   te->segmentMan = new SegmentMan();
+//   te->segmentMan->diskWriter = byteArrayDiskWriter;
+//   te->segmentMan->option = op;
   BtRuntimeHandle btRuntime(new BtRuntime());
   BtRegistry::registerBtRuntime(btContext->getInfoHashAsString(), btRuntime);
 
@@ -107,16 +148,27 @@ DownloadEngineFactory::newTorrentConsoleEngine(const BtContextHandle& btContext,
   BtRegistry::registerBtProgressInfoFile(btContext->getInfoHashAsString(),
 					 btProgressInfoFile);
 
+  BtRegistry::registerPeerObjectCluster(btContext->getInfoHashAsString(),
+					new PeerObjectCluster());
+
+  /*
+  DefaultBtMessageFactoryAdaptorHandle factoryAdaptor =
+    new DefaultBtMessageFactoryAdaptor();
+  BtRegistry::registerBtMessageFactoryAdaptor(btContext->getInfoHashAsString(),
+					      factoryAdaptor);
+
+  BtMessageFactoryClusterHandle factoryCluster = new BtMessageFactoryCluster();
+  BtRegistry::registerBtMessageFactoryCluster(btContext->getInfoHashAsString(),
+					      factoryCluster);
+
+  BtMessageDispatcherClusterHandle dispatcherCluster =
+    new BtMessageDispatcherCluster();
+  BtRegistry::registerBtMessageDispatcherCluster(btContext->getInfoHashAsString(),
+						 dispatcherCluster);
+  */
   te->setBtContext(btContext);
   // initialize file storage
   pieceStorage->initStorage();
-  if(btProgressInfoFile->exists()) {
-    // load .aria2 file if it exists.
-    btProgressInfoFile->load();
-    pieceStorage->getDiskAdaptor()->openExistingFile();
-  } else {
-    pieceStorage->getDiskAdaptor()->initAndOpenFile();
-  }
 
   Integers selectIndexes;
   Util::unfoldRange(op->get(PREF_SELECT_FILE), selectIndexes);
@@ -127,7 +179,8 @@ DownloadEngineFactory::newTorrentConsoleEngine(const BtContextHandle& btContext,
   }
 
   PeerListenCommand* listenCommand =
-    new PeerListenCommand(btRuntime->getNewCuid(), te, btContext);
+    new PeerListenCommand(CUIDCounterSingletonHolder::instance()->newID(),
+			  te, btContext);
   int port;
   int listenPort = op->getAsInt(PREF_LISTEN_PORT);
   if(listenPort == -1) {
@@ -142,24 +195,28 @@ DownloadEngineFactory::newTorrentConsoleEngine(const BtContextHandle& btContext,
   btRuntime->setListenPort(port);
   te->commands.push_back(listenCommand);
   
-  te->commands.push_back(new TrackerWatcherCommand(btRuntime->getNewCuid(),
+  te->commands.push_back(new TrackerWatcherCommand(CUIDCounterSingletonHolder::instance()->newID(),
 						   te,
 						   btContext));
-  te->commands.push_back(new TrackerUpdateCommand(btRuntime->getNewCuid(),
+  te->commands.push_back(new TrackerUpdateCommand(CUIDCounterSingletonHolder::instance()->newID(),
 						  te,
 						  btContext));
-  te->commands.push_back(new TorrentAutoSaveCommand(btRuntime->getNewCuid(),
+  te->commands.push_back(new TorrentAutoSaveCommand(CUIDCounterSingletonHolder::instance()->newID(),
 						    te,
 						    btContext,
 						    op->getAsInt(PREF_AUTO_SAVE_INTERVAL)));
-  te->commands.push_back(new PeerChokeCommand(btRuntime->getNewCuid(),
+  te->commands.push_back(new PeerChokeCommand(CUIDCounterSingletonHolder::instance()->newID(),
 					      te,
 					      btContext,
 					      10));
-  te->commands.push_back(new HaveEraseCommand(btRuntime->getNewCuid(),
+  te->commands.push_back(new HaveEraseCommand(CUIDCounterSingletonHolder::instance()->newID(),
 					      te,
 					      btContext,
 					      10));
+  te->commands.push_back(new ActivePeerConnectionCommand(CUIDCounterSingletonHolder::instance()->newID(),
+							 te,
+							 btContext,
+							 30));
 
   SharedHandle<UnionSeedCriteria> unionCri = new UnionSeedCriteria();
   if(op->defined(PREF_SEED_TIME)) {
@@ -169,7 +226,7 @@ DownloadEngineFactory::newTorrentConsoleEngine(const BtContextHandle& btContext,
     unionCri->addSeedCriteria(new ShareRatioSeedCriteria(op->getAsDouble(PREF_SEED_RATIO), btContext));
   }
   if(unionCri->getSeedCriterion().size() > 0) {
-    te->commands.push_back(new SeedCheckCommand(btRuntime->getNewCuid(),
+    te->commands.push_back(new SeedCheckCommand(CUIDCounterSingletonHolder::instance()->newID(),
 						te,
 						btContext,
 						unionCri));
