@@ -42,11 +42,13 @@
 #include "Util.h"
 
 FtpInitiateConnectionCommand::FtpInitiateConnectionCommand(int cuid,
-							   const RequestHandle req,
+							   const RequestHandle& req,
+							   RequestGroup* requestGroup,
 							   DownloadEngine* e)
-  :AbstractCommand(cuid, req, e)
+  :AbstractCommand(cuid, req, requestGroup, e)
 {
   setTimeout(e->option->getAsInt(PREF_DNS_TIMEOUT));
+  setStatusActive();
   disableReadCheckSocket();
   disableWriteCheckSocket();
 }
@@ -57,14 +59,13 @@ FtpInitiateConnectionCommand::~FtpInitiateConnectionCommand() {
 #endif // ENABLE_ASYNC_DNS
 }
 
-bool FtpInitiateConnectionCommand::executeInternal(Segment& segment) {
+bool FtpInitiateConnectionCommand::executeInternal() {
   string hostname;
   if(useHttpProxy()) {
     hostname = e->option->get(PREF_HTTP_PROXY_HOST);
   } else {
     hostname = req->getHost();
   }
-#ifdef ENABLE_ASYNC_DNS
   if(!Util::isNumbersAndDotsNotation(hostname)) {
     if(resolveHostname(hostname, nameResolver)) {
       hostname = nameResolver->getAddrString();
@@ -73,18 +74,6 @@ bool FtpInitiateConnectionCommand::executeInternal(Segment& segment) {
       return false;
     }
   }
-#endif // ENABLE_ASYNC_DNS
-  if(!e->segmentMan->downloadStarted) {
-    e->segmentMan->filename = Util::urldecode(req->getFile());
-    bool segFileExists = e->segmentMan->segmentFileExists();
-    if(segFileExists) {
-      e->segmentMan->load();
-      e->segmentMan->diskWriter->openExistingFile(e->segmentMan->getFilePath());
-      e->segmentMan->downloadStarted = true;
-    } else {
-      e->segmentMan->diskWriter->initAndOpenFile(e->segmentMan->getFilePath());
-    }
-  }
   Command* command;
   if(useHttpProxy()) {
     logger->info(MSG_CONNECTING_TO_SERVER, cuid,
@@ -94,9 +83,9 @@ bool FtpInitiateConnectionCommand::executeInternal(Segment& segment) {
 				e->option->getAsInt(PREF_HTTP_PROXY_PORT));
     
     if(useHttpProxyGet()) {
-      command = new HttpRequestCommand(cuid, req, e, socket);
+      command = new HttpRequestCommand(cuid, req, _requestGroup, e, socket);
     } else if(useHttpProxyConnect()) {
-      command = new FtpTunnelRequestCommand(cuid, req, e, socket);
+      command = new FtpTunnelRequestCommand(cuid, req, _requestGroup, e, socket);
     } else {
       // TODO
       throw new DlAbortEx("ERROR");
@@ -105,7 +94,7 @@ bool FtpInitiateConnectionCommand::executeInternal(Segment& segment) {
     logger->info(MSG_CONNECTING_TO_SERVER, cuid, req->getHost().c_str(),
 		 req->getPort());
     socket->establishConnection(hostname, req->getPort());
-    command = new FtpNegotiationCommand(cuid, req, e, socket);
+    command = new FtpNegotiationCommand(cuid, req, _requestGroup, e, socket);
   }
   e->commands.push_back(command);
   return true;
