@@ -62,7 +62,10 @@ private:
     SEQ_SEND_RETR,
     SEQ_RECV_RETR,
     SEQ_NEGOTIATION_COMPLETED,
-    SEQ_RETRY
+    SEQ_RETRY,
+    SEQ_HEAD_OK,
+    SEQ_DOWNLOAD_ALREADY_COMPLETED,
+    SEQ_FILE_PREPARATION
   };
   bool recvGreeting();
   bool sendUser();
@@ -79,23 +82,26 @@ private:
   bool recvPort();
   bool sendPasv();
   bool recvPasv();
-  bool sendRest(const Segment& segment);
-  bool sendRestPasv(const Segment& segment);
+  bool sendRest(const SegmentHandle& segment);
+  bool sendRestPasv(const SegmentHandle& segment);
   bool recvRest();
   bool sendRetr();
   bool recvRetr();
-  bool processSequence(const Segment& segment);
+  bool processSequence(const SegmentHandle& segment);
 
   SocketHandle dataSocket;
   SocketHandle serverSocket;
   int sequence;
   FtpConnection* ftp;
 protected:
-  bool executeInternal(Segment& segment);
+  virtual bool executeInternal();
 public:
-  FtpNegotiationCommand(int cuid, const RequestHandle req, DownloadEngine* e,
+  FtpNegotiationCommand(int cuid,
+			const RequestHandle& req,
+			RequestGroup* requestGroup,
+			DownloadEngine* e,
 			const SocketHandle& s);
-  ~FtpNegotiationCommand();
+  virtual ~FtpNegotiationCommand();
 };
 
 #endif // _D_FTP_NEGOTIATION_COMMAND_H_
