@@ -70,6 +70,34 @@ long long int HttpHeader::getFirstAsLLInt(const string& name) const {
   if(value == "") {
     return 0;
   } else {
-    return strtoll(value.c_str(), NULL, 10);
+    return strtoll(value.c_str(), 0, 10);
   }
 }
+
+RangeHandle HttpHeader::getRange() const
+{
+  string rangeStr = getFirst("Content-Range");
+  if(rangeStr == "") {
+    string contentLengthStr = getFirst("Content-Length");
+    if(contentLengthStr == "") {
+      return new Range(0, 0, 0);
+    } else {
+      int64_t contentLength = strtoll(contentLengthStr.c_str(), 0, 10);
+      return new Range(0, contentLength-1, contentLength);
+    }
+  }
+  string::size_type rangeSpecIndex = rangeStr.find("bytes ");
+  if(rangeSpecIndex == string::npos) {
+    return new Range(0, 0, 0);
+  }
+  pair<string, string> rangePair;
+  Util::split(rangePair, rangeStr.substr(rangeSpecIndex+6), '/');
+  pair<string, string> startEndBytePair;
+  Util::split(startEndBytePair, rangePair.first, '-');
+
+  int64_t startByte = STRTOLL(startEndBytePair.first.c_str());
+  int64_t endByte = STRTOLL(startEndBytePair.second.c_str());
+  int64_t entityLength = STRTOLL(rangePair.second.c_str());
+
+  return new Range(startByte, endByte, entityLength);
+}
