@@ -36,7 +36,14 @@
 #include "Util.h"
 #include <algorithm>
 
-MetalinkEntry::MetalinkEntry() {}
+MetalinkEntry::MetalinkEntry():
+  size(0)
+#ifdef ENABLE_MESSAGE_DIGEST
+  ,
+  checksum(0),
+  chunkChecksum(0)
+#endif // ENABLE_MESSAGE_DIGEST
+{}
 
 MetalinkEntry::~MetalinkEntry() {}
 
