@@ -39,24 +39,32 @@
 #include "message.h"
 #include "prefs.h"
 #include "LogFactory.h"
+#include "BitfieldManFactory.h"
+#ifdef ENABLE_MESSAGE_DIGEST
+#include "ChunkChecksumValidator.h"
+#endif // ENABLE_MESSAGE_DIGEST
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <errno.h>
 
-SegmentMan::SegmentMan():bitfield(0),
+SegmentMan::SegmentMan():logger(LogFactory::getInstance()),
+			 bitfield(0),
 			 totalSize(0),
 			 isSplittable(true), 
 			 downloadStarted(false),
 			 dir("."),
 			 errors(0),
-			 diskWriter(0) {
-  logger = LogFactory::getInstance();
-}
+			 diskWriter(0)
+#ifdef ENABLE_MESSAGE_DIGEST
+			,
+			 chunkHashLength(0),
+			 digestAlgo(DIGEST_ALGO_SHA1)
+#endif // ENABLE_MESSAGE_DIGEST
+{}
 
 SegmentMan::~SegmentMan() {
   delete bitfield;
-  delete diskWriter;
 }
 
 bool SegmentMan::segmentFileExists() const {
@@ -93,7 +101,7 @@ void SegmentMan::load() {
 }
 
 void SegmentMan::save() const {
-  if(!isSplittable || totalSize == 0) {
+  if(!isSplittable || totalSize == 0 || !bitfield) {
     return;
   }
   string segFilename = getSegmentFilePath();
@@ -128,7 +136,7 @@ void SegmentMan::save() const {
     }
     for(SegmentEntries::const_iterator itr = usedSegmentEntries.begin();
 	itr != usedSegmentEntries.end(); itr++) {
-      if(fwrite(&(*itr)->segment, sizeof(Segment), 1, segFile) < 1) {
+      if(fwrite((*itr)->segment.get(), sizeof(Segment), 1, segFile) < 1) {
 	throw string("writeError");
       }
     }
@@ -179,8 +187,8 @@ void SegmentMan::read(FILE* file) {
     throw string("readError");
   }
   while(segmentCount--) {
-    Segment seg;
-    if(fread(&seg, sizeof(Segment), 1, file) < 1) {
+    SegmentHandle seg;
+    if(fread(seg.get(), sizeof(Segment), 1, file) < 1) {
       throw string("readError");
     }
     usedSegmentEntries.push_back(SegmentEntryHandle(new SegmentEntry(0, seg)));
@@ -228,71 +236,42 @@ void SegmentMan::init() {
   
 }
 
-void SegmentMan::initBitfield(int segmentLength, long long int totalLength) {
-  this->bitfield = new BitfieldMan(segmentLength, totalLength);
+void SegmentMan::initBitfield(int32_t segmentLength, int64_t totalLength) {
+  delete bitfield;
+  this->bitfield = BitfieldManFactory::getFactoryInstance()->createBitfieldMan(segmentLength, totalLength);
 }
 
-class FindSegmentEntryByIndex {
-private:
-  int index;
-public:
-  FindSegmentEntryByIndex(int index):index(index) {}
-  
-  bool operator()(const SegmentEntryHandle& entry) {
-    return entry->segment.index == index;
-  }
-};
-
-class FindSegmentEntryByCuid {
-private:
-  int cuid;
-public:
-  FindSegmentEntryByCuid(int cuid):cuid(cuid) {}
-
-  bool operator()(const SegmentEntryHandle& entry) {
-    return entry->cuid == cuid;
-  }
-};
-
-Segment SegmentMan::checkoutSegment(int cuid, int index) {
+SegmentHandle SegmentMan::checkoutSegment(int32_t cuid, int32_t index) {
   logger->debug("Attach segment#%d to CUID#%d.", index, cuid);
   bitfield->setUseBit(index);
-
-  SegmentEntries::iterator itr = find_if(usedSegmentEntries.begin(),
-					 usedSegmentEntries.end(),
-					 FindSegmentEntryByIndex(index));
-  Segment segment;
-  if(itr == usedSegmentEntries.end()) {
-    segment = Segment(index, bitfield->getBlockLength(index),
-		       bitfield->getBlockLength());
-    SegmentEntryHandle entry =
-      SegmentEntryHandle(new SegmentEntry(cuid, segment));
+  SegmentEntryHandle segmentEntry = getSegmentEntryByIndex(index);
+  SegmentHandle segment(0);
+  if(segmentEntry.isNull()) {
+    segment = new Segment(index, bitfield->getBlockLength(index),
+			  bitfield->getBlockLength());
+    SegmentEntryHandle entry = new SegmentEntry(cuid, segment);
     usedSegmentEntries.push_back(entry);
   } else {
-    (*itr)->cuid = cuid;
-    segment = (*itr)->segment;
+    segmentEntry->cuid = cuid;
+    segment = segmentEntry->segment;
   }
-
   logger->debug("index=%d, length=%d, segmentLength=%d, writtenLength=%d",
-		segment.index, segment.length, segment.segmentLength,
-		segment.writtenLength);
+		segment->index, segment->length, segment->segmentLength,
+		segment->writtenLength);
   return segment;
 }
 
-bool SegmentMan::onNullBitfield(Segment& segment, int cuid) {
+SegmentHandle SegmentMan::onNullBitfield(int32_t cuid) {
   if(usedSegmentEntries.size() == 0) {
-    segment = Segment(0, 0, 0);
+    SegmentHandle segment = new Segment(0, 0, 0);
     usedSegmentEntries.push_back(SegmentEntryHandle(new SegmentEntry(cuid, segment)));
-    return true;
+    return segment;
   } else {
-    SegmentEntries::iterator itr = find_if(usedSegmentEntries.begin(),
-					   usedSegmentEntries.end(),
-					   FindSegmentEntryByCuid(cuid));
-    if(itr == usedSegmentEntries.end()) {
-      return false;
+    SegmentEntryHandle segmentEntry = getSegmentEntryByCuid(cuid);
+    if(segmentEntry.isNull()) {
+      return 0;
     } else {
-      segment = (*itr)->segment;
-      return true;
+      return segmentEntry->segment;
     }
   }
 }
@@ -301,7 +280,7 @@ SegmentEntryHandle SegmentMan::findSlowerSegmentEntry(const PeerStatHandle& peer
   int speed = (int)(peerStat->getAvgDownloadSpeed()*0.8);
   SegmentEntryHandle slowSegmentEntry(0);
   for(SegmentEntries::const_iterator itr = usedSegmentEntries.begin();
-      itr != usedSegmentEntries.end(); itr++) {
+      itr != usedSegmentEntries.end(); ++itr) {
     const SegmentEntryHandle& segmentEntry = *itr;
     if(segmentEntry->cuid == 0) {
       continue;
@@ -321,112 +300,93 @@ SegmentEntryHandle SegmentMan::findSlowerSegmentEntry(const PeerStatHandle& peer
   return slowSegmentEntry;
 }
 
-bool SegmentMan::getSegment(Segment& segment, int cuid) {
+SegmentHandle SegmentMan::getSegment(int32_t cuid) {
   if(!bitfield) {
-    return onNullBitfield(segment, cuid);
+    return onNullBitfield(cuid);
   }
-  SegmentEntries::iterator itr = find_if(usedSegmentEntries.begin(),
-					 usedSegmentEntries.end(),
-					 FindSegmentEntryByCuid(cuid));
-  if(itr != usedSegmentEntries.end()) {
-    segment = (*itr)->segment;
-    return true;
+  SegmentEntryHandle segmentEntry = getSegmentEntryByCuid(cuid);
+  if(!segmentEntry.isNull()) {
+    return segmentEntry->segment;
   }
   int index = bitfield->getSparseMissingUnusedIndex();
   if(index == -1) {
     PeerStatHandle myPeerStat = getPeerStat(cuid);
     if(!myPeerStat.get()) {
-      return false;
+      return 0;
     }
     SegmentEntryHandle slowSegmentEntry = findSlowerSegmentEntry(myPeerStat);
     if(slowSegmentEntry.get()) {
       logger->info("CUID#%d cancels segment index=%d. CUID#%d handles it instead.",
 		   slowSegmentEntry->cuid,
-		   slowSegmentEntry->segment.index,
+		   slowSegmentEntry->segment->index,
 		   cuid);
       PeerStatHandle slowPeerStat = getPeerStat(slowSegmentEntry->cuid);
       slowPeerStat->requestIdle();
       cancelSegment(slowSegmentEntry->cuid);
-      segment = checkoutSegment(cuid, slowSegmentEntry->segment.index);
-      return true;
+      return checkoutSegment(cuid, slowSegmentEntry->segment->index);
     } else {
-      return false;
+      return 0;
     }
   } else {
-    segment = checkoutSegment(cuid, index);
-    return true;
+    return checkoutSegment(cuid, index);
   }
 }
 
-bool SegmentMan::getSegment(Segment& segment, int cuid, int index) {
+SegmentHandle SegmentMan::getSegment(int32_t cuid, int32_t index) {
   if(!bitfield) {
-    return onNullBitfield(segment, cuid);
+    return onNullBitfield(cuid);
   }
-  if(index < 0 || bitfield->countBlock() <= index) {
-    return false;
+  if(index < 0 || (int32_t)bitfield->countBlock() <= index) {
+    return 0;
   }
   if(bitfield->isBitSet(index) || bitfield->isUseBitSet(index)) {
-    return false;
+    return 0;
   } else {
-    segment = checkoutSegment(cuid, index);
-    return true;
+    return checkoutSegment(cuid, index);
   }
 }
-
+/*
 bool SegmentMan::updateSegment(int cuid, const Segment& segment) {
   if(segment.isNull()) {
     return false;
   }
-  SegmentEntries::iterator itr = find_if(usedSegmentEntries.begin(),
-					 usedSegmentEntries.end(),
-					 FindSegmentEntryByCuid(cuid));
-  if(itr == usedSegmentEntries.end()) {
+  SegmentEntryHandle segmentEntry = getSegmentEntryByCuid(cuid);
+  if(segmentEntry.isNull()) {
     return false;
   } else {
-    (*itr)->segment = segment;
+    segmentEntry->segment = segment;
     return true;
   }
 }
+*/
 
-class CancelSegment {
-private:
-  int cuid;
-  BitfieldMan* bitfield;
-public:
-  CancelSegment(int cuid, BitfieldMan* bitfield):cuid(cuid),
-						 bitfield(bitfield) {}
-  
-  void operator()(SegmentEntryHandle& entry) {
-    if(entry->cuid == cuid) {
-      bitfield->unsetUseBit(entry->segment.index);
-      entry->cuid = 0;
-    }
-  }
-};
-
-void SegmentMan::cancelSegment(int cuid) {
+void SegmentMan::cancelSegment(int32_t cuid) {
   if(bitfield) {
-    for_each(usedSegmentEntries.begin(), usedSegmentEntries.end(),
-	     CancelSegment(cuid, bitfield));
+    for(SegmentEntries::iterator itr = usedSegmentEntries.begin();
+	itr != usedSegmentEntries.end(); ++itr) {
+      if((*itr)->cuid == cuid) {
+	bitfield->unsetUseBit((*itr)->segment->index);
+	(*itr)->cuid = 0;
+	break;
+      }
+    }
   } else {
     usedSegmentEntries.clear();
   }
 }
 
-bool SegmentMan::completeSegment(int cuid, const Segment& segment) {
-  if(segment.isNull()) {
+bool SegmentMan::completeSegment(int32_t cuid, const SegmentHandle& segment) {
+  if(segment->isNull()) {
     return false;
   }
   if(bitfield) {
-    bitfield->unsetUseBit(segment.index);
-    bitfield->setBit(segment.index);
+    bitfield->unsetUseBit(segment->index);
+    bitfield->setBit(segment->index);
   } else {
-    initBitfield(option->getAsInt(PREF_SEGMENT_SIZE), segment.writtenLength);
+    initBitfield(option->getAsInt(PREF_SEGMENT_SIZE), segment->writtenLength);
     bitfield->setAllBit();
   }
-  SegmentEntries::iterator itr = find_if(usedSegmentEntries.begin(),
-					 usedSegmentEntries.end(),
-					 FindSegmentEntryByCuid(cuid));
+  SegmentEntries::iterator itr = getSegmentEntryIteratorByCuid(cuid);
   if(itr == usedSegmentEntries.end()) {
     return false;
   } else {
@@ -435,7 +395,7 @@ bool SegmentMan::completeSegment(int cuid, const Segment& segment) {
   }
 }
 
-bool SegmentMan::hasSegment(int index) const {
+bool SegmentMan::hasSegment(int32_t index) const {
   if(bitfield) {
     return bitfield->isBitSet(index);
   } else {
@@ -443,14 +403,14 @@ bool SegmentMan::hasSegment(int index) const {
   }
 }
 
-long long int SegmentMan::getDownloadLength() const {
-  long long int dlLength = 0;
+int64_t SegmentMan::getDownloadLength() const {
+  int64_t dlLength = 0;
   if(bitfield) {
     dlLength += bitfield->getCompletedLength();
   }
   for(SegmentEntries::const_iterator itr = usedSegmentEntries.begin();
       itr != usedSegmentEntries.end(); itr++) {
-    dlLength += (*itr)->segment.writtenLength;
+    dlLength += (*itr)->segment->writtenLength;
   }
   return dlLength;
 }
@@ -462,7 +422,7 @@ void SegmentMan::registerPeerStat(const PeerStatHandle& peerStat) {
   }
 }
 
-int SegmentMan::calculateDownloadSpeed() const {
+int32_t SegmentMan::calculateDownloadSpeed() const {
   int speed = 0;
   for(PeerStats::const_iterator itr = peerStats.begin();
       itr != peerStats.end(); itr++) {
@@ -473,3 +433,117 @@ int SegmentMan::calculateDownloadSpeed() const {
   }
   return speed;
 }
+
+bool SegmentMan::fileExists() const {
+  return File(getFilePath()).exists();
+}
+
+bool SegmentMan::shouldCancelDownloadForSafety() const {
+  return fileExists() && !segmentFileExists() &&
+    option->get(PREF_ALLOW_OVERWRITE) != V_TRUE;
+}
+
+void SegmentMan::markAllPiecesDone()
+{
+  if(bitfield) {
+    bitfield->setAllBit();
+  }
+}
+
+void SegmentMan::markPieceDone(int64_t length)
+{
+  if(bitfield) {
+    if(length == bitfield->getTotalLength()) {
+      bitfield->setAllBit();
+    } else {
+      bitfield->clearAllBit();
+      int32_t numSegment = length/bitfield->getBlockLength();
+      int32_t remainingLength = length%bitfield->getBlockLength();
+      bitfield->setBitRange(0, numSegment-1);
+      if(remainingLength > 0) {
+	SegmentHandle segment = new Segment();
+	segment->index = numSegment;
+	segment->length = bitfield->getBlockLength(numSegment);
+	segment->segmentLength = bitfield->getBlockLength();
+	segment->writtenLength = remainingLength;
+	usedSegmentEntries.push_back(new SegmentEntry(0, segment));
+      }
+    }
+  }
+}
+
+#ifdef ENABLE_MESSAGE_DIGEST
+void SegmentMan::checkIntegrity()
+{
+  logger->notice("Validating file %s",
+		 getFilePath().c_str());
+  ChunkChecksumValidator v;
+  v.setDigestAlgo(digestAlgo);
+  v.setDiskWriter(diskWriter);
+  v.setFileAllocationMonitor(FileAllocationMonitorFactory::getFactory()->createNewMonitor());
+  v.validate(bitfield, pieceHashes, chunkHashLength);
+}
+#endif // ENABLE_MESSAGE_DIGEST
+
+#ifdef ENABLE_MESSAGE_DIGEST
+bool SegmentMan::isChunkChecksumValidationReady() const {
+  return bitfield && totalSize > 0 &&
+    ((int64_t)pieceHashes.size())*chunkHashLength >= totalSize;
+}
+#endif // ENABLE_MESSAGE_DIGEST
+
+#ifdef ENABLE_MESSAGE_DIGEST
+void SegmentMan::tryChunkChecksumValidation(const SegmentHandle& segment)
+{
+  if(!isChunkChecksumValidationReady()) {
+    return;
+  }
+  int32_t hashStartIndex;
+  int32_t hashEndIndex;
+  Util::indexRange(hashStartIndex, hashEndIndex,
+		   segment->getPosition(),
+		   segment->writtenLength,
+		   chunkHashLength);
+  if(!bitfield->isBitSetOffsetRange((int64_t)hashStartIndex*chunkHashLength,
+				    chunkHashLength)) {
+    ++hashStartIndex;
+  }
+  if(!bitfield->isBitSetOffsetRange((int64_t)hashEndIndex*chunkHashLength,
+				    chunkHashLength)) {
+    --hashEndIndex;
+  }
+  logger->debug("hashStartIndex=%d, hashEndIndex=%d",
+		hashStartIndex, hashEndIndex);
+  if(hashStartIndex > hashEndIndex) {
+    logger->debug("No chunk to verify.");
+    return;
+  }
+  int64_t hashOffset = ((int64_t)hashStartIndex)*chunkHashLength;
+  int32_t startIndex;
+  int32_t endIndex;
+  Util::indexRange(startIndex, endIndex,
+		   hashOffset,
+		   (hashEndIndex-hashStartIndex+1)*chunkHashLength,
+		   bitfield->getBlockLength());
+  logger->debug("startIndex=%d, endIndex=%d", startIndex, endIndex);
+  if(bitfield->isBitRangeSet(startIndex, endIndex)) {
+    for(int32_t index = hashStartIndex; index <= hashEndIndex; ++index) {
+      int64_t offset = ((int64_t)index)*chunkHashLength;
+      int32_t dataLength =
+	offset+chunkHashLength <= totalSize ? chunkHashLength : totalSize-offset;
+      string actualChecksum = diskWriter->messageDigest(offset, dataLength, digestAlgo);
+      string expectedChecksum = pieceHashes[index];
+      if(expectedChecksum == actualChecksum) {
+	logger->info("Good chunk checksum.");
+      } else {
+	logger->info(EX_INVALID_CHUNK_CHECKSUM,
+		     index, offset,
+		     expectedChecksum.c_str(), actualChecksum.c_str());
+	logger->debug("Unset bit from %d to %d(inclusive)", startIndex, endIndex);
+	bitfield->unsetBitRange(startIndex, endIndex);
+	break;
+      }
+    }
+  }
+}
+#endif // ENABLE_MESSAGE_DIGEST
