@@ -32,19 +32,26 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#include "HttpInitiateConnectionCommand.h"
-#include "ConsoleDownloadEngine.h"
-#include "SegmentMan.h"
-#include "LogFactory.h"
 #include "common.h"
-#include "DefaultDiskWriter.h"
+#include "LogFactory.h"
 #include "Util.h"
-#include "InitiateConnectionCommandFactory.h"
 #include "prefs.h"
 #include "FeatureConfig.h"
-#include "DownloadEngineFactory.h"
 #include "UrlRequestInfo.h"
+#include "MultiUrlRequestInfo.h"
 #include "TorrentRequestInfo.h"
+#include "BitfieldManFactory.h"
+#include "SimpleRandomizer.h"
+#include "ConsoleFileAllocationMonitor.h"
+#include "Netrc.h"
+#include "RequestFactory.h"
+#include "OptionParser.h"
+#include "OptionHandlerFactory.h"
+#include "FatalException.h"
+#include "File.h"
+#include "CUIDCounter.h"
+#include "UriFileListParser.h"
+#include "CookieBoxFactory.h"
 #include <deque>
 #include <algorithm>
 #include <time.h>
@@ -52,6 +59,8 @@
 #include <unistd.h>
 #include <libgen.h>
 #include <utility>
+#include <fstream>
+#include <sstream>
 extern char* optarg;
 extern int optind, opterr, optopt;
 #include <getopt.h>
@@ -72,22 +81,12 @@ extern int optind, opterr, optopt;
 
 using namespace std;
 
-bool timeoutSpecified;
-
-void setSignalHander(int signal, void (*handler)(int), int flags) {
-  struct sigaction sigact;
-  sigact.sa_handler = handler;
-  sigact.sa_flags = flags;
-  sigemptyset(&sigact.sa_mask);
-  sigaction(signal, &sigact, NULL);
-}
-
 void showVersion() {
   cout << PACKAGE << _(" version ") << PACKAGE_VERSION << endl;
   cout << "**Configuration**" << endl;
   cout << FeatureConfig::getInstance()->getConfigurationSummary();
   cout << endl;
-  cout << "Copyright (C) 2006 Tatsuhiro Tsujikawa" << endl;
+  cout << "Copyright (C) 2006, 2007 Tatsuhiro Tsujikawa" << endl;
   cout << endl;
   cout <<
     _("This program is free software; you can redistribute it and/or modify\n"
@@ -127,7 +126,8 @@ void showUsage() {
   cout << _(" -s, --split=N                Download a file using N connections. N must be\n"
 	    "                              between 1 and 5. This option affects all URLs.\n"
 	    "                              Thus, aria2 connects to each URL with\n"
-	    "                              N connections.") << endl;
+	    "                              N connections.\n"
+	    "                              Default: 1") << endl;
   cout << _(" --retry-wait=SEC             Set amount of time in second between requests\n"
 	    "                              for errors. Specify a value between 0 and 60.\n"
 	    "                              Default: 5") << endl;
@@ -175,6 +175,45 @@ void showUsage() {
 	    "                              0 means unrestricted.\n"
 	    "                              You can append K or M(1K = 1024, 1M = 1024K).\n"
 	    "                              Default: 0") << endl;
+  cout << _(" --file-allocation=METHOD     Specify file allocation method. METHOD is either\n"
+	    "                              'none' or 'prealloc'.\n"
+	    "                              'none' doesn't pre-allocate file space. 'prealloc'\n"
+	    "                              pre-allocates file space before download begins.\n"
+	    "                              This may take some time depending on the size of\n"
+	    "                              file.\n"
+	    "                              Default: none") << endl;
+  cout << _(" --allow-overwrite=true|false  If this option set to false, aria2 doesn't\n"
+	    "                              download a file which already exists in the file\n"
+	    "                              system but its corresponding .aria2 file doesn't\n"
+	    "                              exist.\n"
+            "                              Default: false") << endl;
+  cout << _(" --check-integrity=true|false  Check file integrity by validating piece hash.\n"
+	    "                              This option makes effect in BitTorrent download\n"
+	    "                              and Metalink with chunk checksums.\n"
+	    "                              Use this option to redownload a damaged portion of\n"
+	    "                              file.\n"
+	    "                              You may need to specify --allow-overwrite=true\n"
+	    "                              option if .aria2 file doesn't exist.\n"
+	    "                              Default: false") << endl;
+  cout << _(" --realtime-chunk-checksum=true|false  Validate chunk checksum while downloading\n"
+	    "                              a file in Metalink mode. This option makes effect\n"
+	    "                              in Metalink with chunk checksums.\n"
+	    "                              Default: true") << endl;
+  cout << _(" -c, --continue               Continue downloading a partially downloaded\n"
+	    "                              file. Use this option to resume a download started\n"
+	    "                              by web browsers or another programs\n"
+	    "                              which download files sequentially from the\n"
+	    "                              beginning. Currently this option is applicable to\n"
+	    "                              http(s)/ftp downloads.") << endl;
+  cout << _(" -U, --user-agent=USER_AGENT  Set user agent for http(s) downloads.") << endl;
+  cout << _(" -n, --no-netrc               Disables netrc support.") << endl;
+  cout << _(" -i, --input-file=FILE        Downloads URIs found in FILE. You can specify\n"
+	    "                              multiple URIs for a single entity: deliminate\n"
+	    "                              URIs by Tab in a single line.") << endl;
+  cout << _(" -j, --max-concurrent-downloads=N Set maximum number of concurrent downloads.\n"
+	    "                              Default: 5") << endl;
+  cout << _(" --load-cookies=FILE          Load cookies from FILE. The format of FILE is\n"
+	    "                              one used by Netscape and Mozilla.") << endl;
 #ifdef ENABLE_BITTORRENT
   cout << _(" -T, --torrent-file=TORRENT_FILE  The file path to .torrent file.") << endl;
   cout << _(" --follow-torrent=true|false  Setting this option to false prevents aria2 to\n"
@@ -207,9 +246,8 @@ void showUsage() {
 #ifdef ENABLE_METALINK
   cout << _(" -M, --metalink-file=METALINK_FILE The file path to .metalink file.") << endl;
   cout << _(" -C, --metalink-servers=NUM_SERVERS The number of servers to connect to\n"
-	    "                              simultaneously. If more than one connection per\n"
-	    "                              server is required, use -s option.\n"
-	    "                              Default: 15") << endl;
+	    "                              simultaneously.\n"
+	    "                              Default: 5") << endl;
   cout << _(" --metalink-version=VERSION   The version of file to download.") << endl;
   cout << _(" --metalink-language=LANGUAGE The language of file to download.") << endl;
   cout << _(" --metalink-os=OS             The operating system the file is targeted.") << endl;
@@ -266,21 +304,6 @@ void showUsage() {
   cout << endl;
 }
 
-long long int getRealSize(char* optarg) {
-  string::size_type p = string(optarg).find_first_of("KM");
-  int mult = 1;
-  if(p != string::npos) {
-    if(optarg[p] == 'K') {
-      mult = 1024;
-    } else if(optarg[p] == 'M') {
-      mult = 1024*1024;
-    }
-    optarg[p] = '\0';
-  }
-  long long int size = strtoll(optarg, NULL, 10)*mult;
-  return size;
-}
-
 int main(int argc, char* argv[]) {
 #ifdef ENABLE_NLS
   setlocale (LC_CTYPE, "");
@@ -288,8 +311,7 @@ int main(int argc, char* argv[]) {
   bindtextdomain (PACKAGE, LOCALEDIR);
   textdomain (PACKAGE);
 #endif // ENABLE_NLS
-  timeoutSpecified = false;
-
+  stringstream cmdstream;
   int c;
   Option* op = new Option();
   op->put(PREF_STDOUT_LOG, V_FALSE);
@@ -299,7 +321,7 @@ int main(int argc, char* argv[]) {
   op->put(PREF_SEGMENT_SIZE, Util::itos(1024*1024));
   op->put(PREF_HTTP_KEEP_ALIVE, V_FALSE);
   op->put(PREF_LISTEN_PORT, "-1");
-  op->put(PREF_METALINK_SERVERS, "15");
+  op->put(PREF_METALINK_SERVERS, "5");
   op->put(PREF_FOLLOW_TORRENT,
 #ifdef ENABLE_BITTORRENT
 	  V_TRUE
@@ -318,12 +340,13 @@ int main(int argc, char* argv[]) {
   op->put(PREF_TIMEOUT, "60");
   op->put(PREF_DNS_TIMEOUT, "10");
   op->put(PREF_PEER_CONNECTION_TIMEOUT, "60");
+  op->put(PREF_BT_TIMEOUT, "180");
+  op->put(PREF_BT_REQUEST_TIMEOUT, "60");
+  op->put(PREF_BT_KEEP_ALIVE_INTERVAL, "120");
   op->put(PREF_MIN_SEGMENT_SIZE, "1048576");// 1M
   op->put(PREF_MAX_TRIES, "5");
   op->put(PREF_HTTP_AUTH_SCHEME, V_BASIC);
   op->put(PREF_HTTP_PROXY_METHOD, V_TUNNEL);
-  op->put(PREF_FTP_USER, "anonymous");
-  op->put(PREF_FTP_PASSWD, "ARIA2USER@");
   op->put(PREF_FTP_TYPE, V_BINARY);
   op->put(PREF_FTP_VIA_HTTP_PROXY, V_TUNNEL);
   op->put(PREF_AUTO_SAVE_INTERVAL, "60");
@@ -333,6 +356,16 @@ int main(int argc, char* argv[]) {
   op->put(PREF_MAX_UPLOAD_LIMIT, "0");
   op->put(PREF_STARTUP_IDLE_TIME, "10");
   op->put(PREF_TRACKER_MAX_TRIES, "10");
+  op->put(PREF_FILE_ALLOCATION, V_NONE);
+  op->put(PREF_ALLOW_OVERWRITE, V_FALSE);
+  op->put(PREF_REALTIME_CHUNK_CHECKSUM, V_TRUE);
+  op->put(PREF_CHECK_INTEGRITY, V_FALSE);
+  op->put(PREF_NETRC_PATH, Util::getHomeDir()+"/.netrc");
+  op->put(PREF_CONTINUE, V_FALSE);
+  op->put(PREF_USER_AGENT, "aria2");
+  op->put(PREF_NO_NETRC, V_FALSE);
+  op->put(PREF_MAX_CONCURRENT_DOWNLOADS, "5");
+  op->put(PREF_DIRECT_DOWNLOAD_TIMEOUT, "15");
   while(1) {
     int optIndex = 0;
     int lopt;
@@ -361,6 +394,16 @@ int main(int argc, char* argv[]) {
       { "http-proxy-method", required_argument, &lopt, 14 },
       { "lowest-speed-limit", required_argument, &lopt, 200 },
       { "max-download-limit", required_argument, &lopt, 201 },
+      { "file-allocation", required_argument, 0, 'a' },
+      { "allow-overwrite", required_argument, &lopt, 202 },
+      { "check-integrity", required_argument, &lopt, 203 },
+      { "realtime-chunk-checksum", required_argument, &lopt, 204 },
+      { "continue", no_argument, 0, 'c' },
+      { "user-agent", required_argument, 0, 'U' },
+      { "no-netrc", no_argument, 0, 'n' },
+      { "input-file", required_argument, 0, 'i' },
+      { "max-concurrent-downloads", required_argument, 0, 'j' },
+      { "load-cookies", required_argument, &lopt, 205 },
 #ifdef ENABLE_BITTORRENT
       { "torrent-file", required_argument, NULL, 'T' },
       { "listen-port", required_argument, &lopt, 15 },
@@ -369,7 +412,7 @@ int main(int argc, char* argv[]) {
       { "no-preallocation", no_argument, &lopt, 18 },
       { "direct-file-mapping", required_argument, &lopt, 19 },
       // TODO remove upload-limit.
-      { "upload-limit", required_argument, &lopt, 20 },
+      //{ "upload-limit", required_argument, &lopt, 20 },
       { "select-file", required_argument, &lopt, 21 },
       { "seed-time", required_argument, &lopt, 22 },
       { "seed-ratio", required_argument, &lopt, 23 },
@@ -388,302 +431,169 @@ int main(int argc, char* argv[]) {
       { "help", no_argument, NULL, 'h' },
       { 0, 0, 0, 0 }
     };
-    c = getopt_long(argc, argv, "Dd:o:l:s:pt:m:vhST:M:C:", longOpts, &optIndex);
+    c = getopt_long(argc, argv, "Dd:o:l:s:pt:m:vhST:M:C:a:cU:ni:j:", longOpts, &optIndex);
     if(c == -1) {
       break;
     }
     switch(c) {
     case 0:{
       switch(lopt) {
-      case 1: {
-	pair<string, string> proxy;
-	Util::split(proxy, optarg, ':');
-	int port = (int)strtol(proxy.second.c_str(), NULL, 10);
-	if(proxy.first.empty() || proxy.second.empty() ||
-	   !(0 < port && port <= 65535)) {
-	  cerr << _("unrecognized proxy format") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_HTTP_PROXY_HOST, proxy.first);
-	op->put(PREF_HTTP_PROXY_PORT, Util::itos(port));
-	op->put(PREF_HTTP_PROXY_ENABLED, V_TRUE);
+      case 1:
+	cmdstream << PREF_HTTP_PROXY << "=" << optarg << "\n";
 	break;
-      }
       case 2:
-	op->put(PREF_HTTP_USER, optarg);
-	op->put(PREF_HTTP_AUTH_ENABLED, V_TRUE);
+	cmdstream << PREF_HTTP_USER << "=" << optarg << "\n";
 	break;
       case 3:
-	op->put(PREF_HTTP_PASSWD, optarg);
+	cmdstream << PREF_HTTP_PASSWD << "=" << optarg << "\n";
 	break;
       case 4:
-	op->put(PREF_HTTP_PROXY_USER, optarg);
-	op->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
+	cmdstream << PREF_HTTP_PROXY_USER << "=" << optarg << "\n";
 	break;
       case 5: 
-	op->put(PREF_HTTP_PROXY_PASSWD, optarg);
+	cmdstream << PREF_HTTP_PROXY_PASSWD << "=" << optarg << "\n";
 	break;
       case 6:
-	if(string(V_BASIC) == optarg) {
-	  op->put(PREF_HTTP_AUTH_SCHEME, V_BASIC);
-	} else {
-	  cerr << _("Currently, supported authentication scheme is basic.") << endl;
-	}
+	cmdstream << PREF_HTTP_AUTH_SCHEME << "=" << optarg << "\n";
 	break;
       case 7:
-	op->put(PREF_REFERER, optarg);
+	cmdstream << PREF_REFERER << "=" << optarg << "\n";
 	break;
-      case 8: {
-	int wait = (int)strtol(optarg, NULL, 10);
-	if(!(0 <= wait && wait <= 60)) {
-	  cerr << _("retry-wait must be between 0 and 60.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_RETRY_WAIT, Util::itos(wait));
+      case 8:
+	cmdstream << PREF_RETRY_WAIT << "=" << optarg << "\n";
 	break;
-      }
       case 9:
-	op->put(PREF_FTP_USER, optarg);
+	cmdstream << PREF_FTP_USER << "=" << optarg << "\n";
 	break;
       case 10:
-	op->put(PREF_FTP_PASSWD, optarg);
+	cmdstream << PREF_FTP_PASSWD << "=" << optarg << "\n";
 	break;
       case 11:
-	if(string(optarg) == V_BINARY || string(optarg) == V_ASCII) {
-	  op->put(PREF_FTP_TYPE, optarg);
-	} else {
-	  cerr << _("ftp-type must be either 'binary' or 'ascii'.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
+	cmdstream << PREF_FTP_TYPE << "=" << optarg << "\n";
 	break;
       case 12:
-	if(string(optarg) == V_GET || string(optarg) == V_TUNNEL) {
-	  op->put(PREF_FTP_VIA_HTTP_PROXY, optarg);
-	} else {
-	  cerr << _("ftp-via-http-proxy must be either 'get' or 'tunnel'.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
+	cmdstream << PREF_FTP_VIA_HTTP_PROXY << "=" << optarg << "\n";
 	break;
-      case 13: {
-	long long int size = getRealSize(optarg);
-	if(size < 1024) {
-	  cerr << _("min-segment-size invalid") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_MIN_SEGMENT_SIZE, Util::llitos(size));
+      case 13:
+	cmdstream << PREF_MIN_SEGMENT_SIZE << "=" << optarg << "\n";
 	break;
-      }
       case 14:
-	if(string(optarg) == V_GET || string(optarg) == V_TUNNEL) {
-	  op->put(PREF_HTTP_PROXY_METHOD, optarg);
-	} else {
-	  cerr << _("http-proxy-method must be either 'get' or 'tunnel'.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
+	cmdstream << PREF_HTTP_PROXY_METHOD << "=" << optarg << "\n";
 	break;
-      case 15: {
-	int listenPort = (int)strtol(optarg, NULL, 10);
-	if(!(1024 <= listenPort && listenPort <= 65535)) {
-	  cerr << _("listen-port must be between 1024 and 65535.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_LISTEN_PORT, Util::itos(listenPort));
+      case 15:
+	cmdstream << PREF_LISTEN_PORT << "=" << optarg << "\n";
 	break;
-      }
       case 16:
-	if(string(optarg) == "true") {
-	  op->put(PREF_FOLLOW_TORRENT, V_TRUE);
-	} else if(string(optarg) == "false") {
-	  op->put(PREF_FOLLOW_TORRENT, V_FALSE);
-	} else {
-	  cerr << _("follow-torrent must be either 'true' or 'false'.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
+	cmdstream << PREF_FOLLOW_TORRENT << "=" << optarg << "\n";
 	break;
       case 18:
-	op->put(PREF_NO_PREALLOCATION, V_TRUE);
+	cmdstream << PREF_NO_PREALLOCATION << "=" << V_TRUE << "\n";
 	break;
       case 19:
-	if(string(optarg) == "true") {
-	  op->put(PREF_DIRECT_FILE_MAPPING, V_TRUE);
-	} else if(string(optarg) == "false") {
-	  op->put(PREF_DIRECT_FILE_MAPPING, V_FALSE);
-	} else {
-	  cerr << _("direct-file-mapping must be either 'true' or 'false'.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
+	cmdstream << PREF_DIRECT_FILE_MAPPING << "=" << optarg << "\n";
 	break;
-      case 20: {
-	cerr << "Warning: upload-limit will be deprecated in the future release.\n"
-	  "Use max-upload-limit instead. Because there is a difference between them,\n"
-	  "take a look at the description of max-upload-limit option." << endl;
-	int uploadSpeed = strtol(optarg, NULL, 10)*1024;
-	if(0 > uploadSpeed) {
-	  cerr << _("upload-limit must be greater than or equal to 0.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_MAX_UPLOAD_LIMIT, Util::itos(uploadSpeed));
-	break;
-      }
       case 21:
-	op->put(PREF_SELECT_FILE, optarg);
+	cmdstream << PREF_SELECT_FILE << "=" << optarg << "\n";
 	break;
-      case 22: {
-	int seedTime = (int)strtol(optarg, NULL, 10);
-	if(seedTime < 0) {
-	  cerr << _("seed-time must be greater than or equal to 0.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_SEED_TIME, Util::itos(seedTime));
+      case 22:
+	cmdstream << PREF_SEED_TIME << "=" << optarg << "\n";
 	break;
-      }
-      case 23: {
-	double ratio = (int)strtod(optarg, NULL);
-	if(ratio < 0.0) {
-	  cerr << _("seed-ratio must be greater than or equal to 0.0.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_SEED_RATIO, optarg);
+      case 23:
+	cmdstream << PREF_SEED_RATIO << "=" << optarg << "\n";
 	break;
-      }
-      case 24: {
-	int limit = getRealSize(optarg);
-	if(limit < 0) {
-	  cerr << _("max-upload-limit must be greater than or equal to 0") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_MAX_UPLOAD_LIMIT, Util::itos(limit));
+      case 24:
+	cmdstream << PREF_MAX_UPLOAD_LIMIT << "=" << optarg << "\n";
 	break;
-      }
       case 100:
-	op->put(PREF_METALINK_VERSION, optarg);
+	cmdstream << PREF_METALINK_VERSION << "=" << optarg << "\n";
 	break;
       case 101:
-	op->put(PREF_METALINK_LANGUAGE, optarg);
+	cmdstream << PREF_METALINK_LANGUAGE << "=" << optarg << "\n";
 	break;
       case 102:
-	op->put(PREF_METALINK_OS, optarg);
+	cmdstream << PREF_METALINK_OS << "=" << optarg << "\n";
 	break;
       case 103:
-	if(string(optarg) == "true") {
-	  op->put(PREF_FOLLOW_METALINK, V_TRUE);
-	} else if(string(optarg) == "false") {
-	  op->put(PREF_FOLLOW_METALINK, V_FALSE);
-	} else {
-	  cerr << _("follow-metalink must be either 'true' or 'false'.") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
+	cmdstream << PREF_FOLLOW_METALINK << "=" << optarg << "\n";
 	break;
       case 104:
-	op->put(PREF_METALINK_LOCATION, optarg);
+	cmdstream << PREF_METALINK_LOCATION << "=" << optarg << "\n";
 	break;
-      case 200: {
-	int limit = getRealSize(optarg);
-	if(limit < 0) {
-	  cerr << _("lowest-speed-limit must be greater than or equal to 0") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_LOWEST_SPEED_LIMIT, Util::itos(limit));
+      case 200:
+	cmdstream << PREF_LOWEST_SPEED_LIMIT << "=" << optarg << "\n";
 	break;
-      }
-      case 201: {
-	int limit = getRealSize(optarg);
-	if(limit < 0) {
-	  cerr << _("max-download-limit must be greater than or equal to 0") << endl;
-	  showUsage();
-	  exit(EXIT_FAILURE);
-	}
-	op->put(PREF_MAX_DOWNLOAD_LIMIT, Util::itos(limit));
+      case 201:
+	cmdstream << PREF_MAX_DOWNLOAD_LIMIT << "=" << optarg << "\n";
+	break;
+      case 202:
+	cmdstream << PREF_ALLOW_OVERWRITE << "=" << optarg << "\n";
+	break;
+      case 203:
+	cmdstream << PREF_CHECK_INTEGRITY << "=" << optarg << "\n";
+	break;
+      case 204:
+	cmdstream << PREF_REALTIME_CHUNK_CHECKSUM << "=" << optarg << "\n";
+	break;
+      case 205:
+	cmdstream << PREF_LOAD_COOKIES << "=" << optarg << "\n";
 	break;
-      }
       }
       break;
     }
     case 'D':
-      op->put(PREF_DAEMON, V_TRUE);
+      cmdstream << PREF_DAEMON << "=" << V_TRUE << "\n";
       break;
     case 'd':
-      op->put(PREF_DIR, optarg);
+      cmdstream << PREF_DIR << "=" << optarg << "\n";
       break;
     case 'o':
-      op->put(PREF_OUT, optarg);
+      cmdstream << PREF_OUT << "=" << optarg << "\n";
       break;
     case 'l':
-      if(strcmp("-", optarg) == 0) {
-	op->put(PREF_STDOUT_LOG, V_TRUE);
-      } else {
-	op->put(PREF_LOG, optarg);
-      }
+      cmdstream << PREF_LOG << "=" << optarg << "\n";
       break;
-    case 's': {
-      int split = (int)strtol(optarg, NULL, 10);
-      if(!(1 <= split && split <= 5)) {
-	cerr << _("split must be between 1 and 5.") << endl;
-	showUsage();
-	exit(EXIT_FAILURE);
-      }
-      op->put(PREF_SPLIT, Util::itos(split));
+    case 's':
+      cmdstream << PREF_SPLIT << "=" << optarg << "\n";
       break;
-    }
-    case 't': {
-      int timeout = (int)strtol(optarg, NULL, 10);
-      if(1 <= timeout && timeout <= 600) {
-	op->put(PREF_TIMEOUT, Util::itos(timeout));
-	timeoutSpecified = true;
-      } else {
-	cerr << _("timeout must be between 1 and 600") << endl;
-	showUsage();
-	exit(EXIT_FAILURE);
-      }
+    case 't':
+      cmdstream << PREF_TIMEOUT << "=" << optarg << "\n";
       break;
-    }
-    case 'm': {
-      int retries = (int)strtol(optarg, NULL, 10);
-      if(retries < 0) {
-	cerr << _("max-tries invalid") << endl;
-	showUsage();
-	exit(EXIT_FAILURE);
-      }
-      op->put(PREF_MAX_TRIES, Util::itos(retries));
+    case 'm':
+      cmdstream << PREF_MAX_TRIES << "=" << optarg << "\n";
       break;
-    }
     case 'p':
-      op->put(PREF_FTP_PASV_ENABLED, V_TRUE);
+      cmdstream << PREF_FTP_PASV << "=" << V_TRUE << "\n";
       break;
     case 'S':
-      op->put(PREF_SHOW_FILES, V_TRUE);
+      cmdstream << PREF_SHOW_FILES << "=" << V_TRUE << "\n";
       break;
     case 'T':
-      op->put(PREF_TORRENT_FILE, optarg);
+      cmdstream << PREF_TORRENT_FILE << "=" << optarg << "\n";
       break;
     case 'M':
-      op->put(PREF_METALINK_FILE, optarg);
+      cmdstream << PREF_METALINK_FILE << "=" << optarg << "\n";
       break;
-    case 'C': {
-      int metalinkServers = (int)strtol(optarg, NULL, 10);
-      if(metalinkServers <= 0) {
-	cerr << _("metalink-servers must be greater than 0.") << endl;
-	showUsage();
-	exit(EXIT_FAILURE);
-      }
-      op->put(PREF_METALINK_SERVERS, Util::itos(metalinkServers));
+    case 'C':
+      cmdstream << PREF_METALINK_SERVERS << "=" << optarg << "\n";
+      break;
+    case 'a':
+      cmdstream << PREF_FILE_ALLOCATION << "=" << optarg << "\n";
+      break;
+    case 'c':
+      cmdstream << PREF_CONTINUE << "=" << V_TRUE << "\n";
+      break;
+    case 'U':
+      cmdstream << PREF_USER_AGENT << "=" << optarg << "\n";
+      break;
+    case 'n':
+      cmdstream << PREF_NO_NETRC << "=" << V_TRUE << "\n";
+      break;
+    case 'i':
+      cmdstream << PREF_INPUT_FILE << "=" << optarg << "\n";
+      break;
+    case 'j':
+      cmdstream << PREF_MAX_CONCURRENT_DOWNLOADS << "=" << optarg << "\n";
       break;
-    }
     case 'v':
       showVersion();
       exit(EXIT_SUCCESS);
@@ -691,14 +601,41 @@ int main(int argc, char* argv[]) {
       showUsage();
       exit(EXIT_SUCCESS);
     default:
-      showUsage();
       exit(EXIT_FAILURE);
     }
   }
-  if(!op->defined(PREF_TORRENT_FILE) && !op->defined(PREF_METALINK_FILE)) {
+
+  {
+    OptionParser oparser;
+    oparser.setOptionHandlers(OptionHandlerFactory::createOptionHandlers());
+    string cfname = Util::getHomeDir()+"/.aria2/aria2.conf";
+    ifstream cfstream(cfname.c_str());
+    try {
+      oparser.parse(op, cfstream);
+    } catch(Exception* e) {
+      cerr << "Parse error in " << cfname << endl;
+      cerr << e->getMsg() << endl;
+      delete e;
+      exit(EXIT_FAILURE);
+    }
+    try {
+      oparser.parse(op, cmdstream);
+    } catch(Exception* e) {
+      cerr << e->getMsg() << endl;
+      delete e;
+      exit(EXIT_FAILURE);
+    }
+  }
+  if(op->defined(PREF_HTTP_USER)) {
+    op->put(PREF_HTTP_AUTH_ENABLED, V_TRUE);
+  }
+  if(op->defined(PREF_HTTP_PROXY_USER)) {
+    op->put(PREF_HTTP_PROXY_AUTH_ENABLED, V_TRUE);
+  }
+  if(!op->defined(PREF_TORRENT_FILE) && !op->defined(PREF_METALINK_FILE) &&
+     !op->defined(PREF_INPUT_FILE)) {
     if(optind == argc) {
       cerr << _("specify at least one URL") << endl;
-      showUsage();
       exit(EXIT_FAILURE);
     }
   }
@@ -721,7 +658,9 @@ int main(int argc, char* argv[]) {
 #ifdef ENABLE_METALINK
   xmlInitParser();
 #endif // ENABLE_METALINK
-  srandom(time(NULL));
+  SimpleRandomizer::init();
+  BitfieldManFactory::setDefaultRandomizer(SimpleRandomizer::getInstance());
+  FileAllocationMonitorFactory::setFactory(new ConsoleFileAllocationMonitorFactory());
   if(op->getAsBool(PREF_STDOUT_LOG)) {
     LogFactory::setLogFile("/dev/stdout");
   } else if(op->get(PREF_LOG).size()) {
@@ -735,7 +674,39 @@ int main(int argc, char* argv[]) {
     logger->info("%s %s", PACKAGE, PACKAGE_VERSION);
     logger->info("Logging started.");
 
-    setSignalHander(SIGPIPE, SIG_IGN, 0);
+    RequestFactoryHandle requestFactory = new RequestFactory();
+    requestFactory->setOption(op);
+    File netrccf(op->get(PREF_NETRC_PATH));
+    if(!op->getAsBool(PREF_NO_NETRC) && netrccf.isFile()) {
+      mode_t mode = netrccf.mode();
+      if(mode&(S_IRWXG|S_IRWXO)) {
+	logger->notice(".netrc file %s does not have correct permissions. It should be 600. netrc support disabled.",
+		       op->get(PREF_NETRC_PATH).c_str());
+      } else {
+	NetrcHandle netrc = new Netrc();
+	netrc->parse(op->get(PREF_NETRC_PATH));
+	requestFactory->setNetrc(netrc);
+      }
+    }
+
+    CookieBoxFactoryHandle cookieBoxFactory = new CookieBoxFactory();
+    CookieBoxFactorySingletonHolder::instance(cookieBoxFactory);
+    if(op->defined(PREF_LOAD_COOKIES)) {
+      File cookieFile(op->get(PREF_LOAD_COOKIES));
+      if(cookieFile.isFile()) {
+	ifstream in(op->get(PREF_LOAD_COOKIES).c_str());
+	CookieBoxFactorySingletonHolder::instance()->loadDefaultCookie(in);
+      } else {
+	logger->error("Failed to load cookies from %s", op->get(PREF_LOAD_COOKIES).c_str());
+	exit(EXIT_FAILURE);
+      }
+    }
+
+    RequestFactorySingletonHolder::instance(requestFactory);
+    CUIDCounterHandle cuidCounter = new CUIDCounter();
+    CUIDCounterSingletonHolder::instance(cuidCounter);
+
+    Util::setGlobalSignalHandler(SIGPIPE, SIG_IGN, 0);
 
     RequestInfo* firstReqInfo = 0;
 #ifdef ENABLE_BITTORRENT
@@ -753,12 +724,26 @@ int main(int argc, char* argv[]) {
 #ifdef ENABLE_METALINK
       if(op->defined(PREF_METALINK_FILE)) {
 	firstReqInfo = new MetalinkRequestInfo(op->get(PREF_METALINK_FILE),
-					      op);
-      } else
+					       op);
+      }
+      else
 #endif // ENABLE_METALINK
-	{
-	  firstReqInfo = new UrlRequestInfo(args, 0, op);
+	if(op->defined(PREF_INPUT_FILE)) {
+	  UriFileListParser flparser(op->get(PREF_INPUT_FILE));
+	  RequestGroups groups;
+	  while(flparser.hasNext()) {
+	    Strings uris = flparser.next();
+	    if(!uris.empty()) {
+	      RequestGroupHandle rg = new RequestGroup(uris, op);
+	      groups.push_back(rg);
+	    }
+	  }
+	  firstReqInfo = new MultiUrlRequestInfo(groups, op);
 	}
+	else
+	  {
+	    firstReqInfo = new MultiUrlRequestInfo(args, op);
+	  }
 
     RequestInfos reqInfos;
     if(firstReqInfo) {
@@ -769,6 +754,7 @@ int main(int argc, char* argv[]) {
       reqInfos.pop_front();
       RequestInfos nextReqInfos = reqInfo->execute();
       copy(nextReqInfos.begin(), nextReqInfos.end(), front_inserter(reqInfos));
+      /*
       if(reqInfo->isFail()) {
 	exitStatus = EXIT_FAILURE;
       } else if(reqInfo->getFileInfo().checkReady()) {
@@ -783,9 +769,10 @@ int main(int argc, char* argv[]) {
 	  exitStatus = EXIT_FAILURE;
 	}
       }
+      */
     }
   } catch(Exception* ex) {
-    cerr << ex->getMsg() << endl;
+    cerr << "Exception caught:\n" << ex->getMsg() << endl;
     delete ex;
     exit(EXIT_FAILURE);
   }
