@@ -2,6 +2,8 @@
 #include "DefaultBtContext.h"
 #include "Util.h"
 #include "Exception.h"
+#include "FixedNumberRandomizer.h"
+#include "BitfieldManFactory.h"
 #include <cppunit/extensions/HelperMacros.h>
 
 using namespace std;
@@ -14,13 +16,20 @@ class DefaultPieceStorageTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testGetMissingFastPiece);
   CPPUNIT_TEST(testHasMissingPiece);
   CPPUNIT_TEST(testCompletePiece);
+  CPPUNIT_TEST(testGetPiece);
+  CPPUNIT_TEST(testGetPieceInUsedPieces);
+  CPPUNIT_TEST(testGetPieceCompletedPiece);
   CPPUNIT_TEST_SUITE_END();
 private:
   BtContextHandle btContext;
   PeerHandle peer;
   Option* option;
 public:
-  DefaultPieceStorageTest():btContext(0) {}
+  DefaultPieceStorageTest():btContext(0), peer(0) {
+    FixedNumberRandomizer* randomizer = new FixedNumberRandomizer();
+    randomizer->setFixedNumber(0);
+    BitfieldManFactory::setDefaultRandomizer(randomizer);
+  }
 
   void setUp() {
     btContext = BtContextHandle(new DefaultBtContext());
@@ -36,6 +45,9 @@ public:
   void testGetMissingFastPiece();
   void testHasMissingPiece();
   void testCompletePiece();
+  void testGetPiece();
+  void testGetPieceInUsedPieces();
+  void testGetPieceCompletedPiece();
 };
 
 
@@ -53,17 +65,17 @@ void DefaultPieceStorageTest::testGetMissingPiece() {
   pss.setEndGamePieceNum(0);
 
   peer->setAllBitfield();
-  Piece piece = pss.getMissingPiece(peer);
+  PieceHandle piece = pss.getMissingPiece(peer);
   CPPUNIT_ASSERT_EQUAL(string("piece: index=0, length=128"),
-		       piece.toString());
+		       piece->toString());
   piece = pss.getMissingPiece(peer);
   CPPUNIT_ASSERT_EQUAL(string("piece: index=1, length=128"),
-		       piece.toString());
+		       piece->toString());
   piece = pss.getMissingPiece(peer);
   CPPUNIT_ASSERT_EQUAL(string("piece: index=2, length=128"),
-		       piece.toString());
+		       piece->toString());
   piece = pss.getMissingPiece(peer);
-  CPPUNIT_ASSERT(Piece::isNull(piece));
+  CPPUNIT_ASSERT(piece.isNull());
 }
 
 void DefaultPieceStorageTest::testGetMissingFastPiece() {
@@ -74,9 +86,9 @@ void DefaultPieceStorageTest::testGetMissingFastPiece() {
   peer->setFastExtensionEnabled(true);
   peer->addFastSetIndex(2);
 
-  Piece piece = pss.getMissingFastPiece(peer);
+  PieceHandle piece = pss.getMissingFastPiece(peer);
   CPPUNIT_ASSERT_EQUAL(string("piece: index=2, length=128"),
-		       piece.toString());
+		       piece->toString());
 }
 
 void DefaultPieceStorageTest::testHasMissingPiece() {
@@ -95,9 +107,9 @@ void DefaultPieceStorageTest::testCompletePiece() {
 
     peer->setAllBitfield();
 
-  Piece piece = pss.getMissingPiece(peer);
+  PieceHandle piece = pss.getMissingPiece(peer);
   CPPUNIT_ASSERT_EQUAL(string("piece: index=0, length=128"),
-		       piece.toString());
+		       piece->toString());
 
   CPPUNIT_ASSERT_EQUAL((long long int)0,
 		       pss.getCompletedLength());
@@ -108,3 +120,33 @@ void DefaultPieceStorageTest::testCompletePiece() {
 		       pss.getCompletedLength());
 
 }
+
+void DefaultPieceStorageTest::testGetPiece() {
+  DefaultPieceStorage pss(btContext, option);
+  
+  PieceHandle pieceGot = pss.getPiece(0);
+  CPPUNIT_ASSERT_EQUAL(0, pieceGot->getIndex());
+  CPPUNIT_ASSERT_EQUAL(128, pieceGot->getLength());
+  CPPUNIT_ASSERT_EQUAL(false, pieceGot->pieceComplete());
+}
+
+void DefaultPieceStorageTest::testGetPieceInUsedPieces() {
+  DefaultPieceStorage pss(btContext, option);
+  PieceHandle piece = PieceHandle(new Piece(0, 128));
+  piece->completeBlock(0);
+  pss.addUsedPiece(piece);
+  PieceHandle pieceGot = pss.getPiece(0);
+  CPPUNIT_ASSERT_EQUAL(0, pieceGot->getIndex());
+  CPPUNIT_ASSERT_EQUAL(128, pieceGot->getLength());
+  CPPUNIT_ASSERT_EQUAL(1, pieceGot->countCompleteBlock());
+}
+
+void DefaultPieceStorageTest::testGetPieceCompletedPiece() {
+  DefaultPieceStorage pss(btContext, option);
+  PieceHandle piece = PieceHandle(new Piece(0, 128));
+  pss.completePiece(piece);
+  PieceHandle pieceGot = pss.getPiece(0);
+  CPPUNIT_ASSERT_EQUAL(0, pieceGot->getIndex());
+  CPPUNIT_ASSERT_EQUAL(128, pieceGot->getLength());
+  CPPUNIT_ASSERT_EQUAL(true, pieceGot->pieceComplete());
+}
