@@ -21,6 +21,11 @@ public:
     copy(peers.begin(), peers.end(), back_inserter(this->peers));
   }
 
+  virtual bool addIncomingPeer(const PeerHandle& peer)
+  {
+    return true;
+  }
+
   virtual const Peers& getPeers() {
     return peers;
   }
@@ -44,6 +49,12 @@ public:
   void setStat(const TransferStat& stat) {
     this->stat = stat;
   }
+
+  virtual void returnPeer(const PeerHandle& peer)
+  {
+  }
 };
 
+typedef SharedHandle<MockPeerStorage> MockPeerStorageHandle;
+
 #endif // _D_MOCK_PEER_STORAGE_H_
