@@ -1,5 +1,7 @@
 #include "Request.h"
-
+#include "Netrc.h"
+#include "DefaultAuthResolver.h"
+#include "NetrcAuthResolver.h"
 #include <cppunit/extensions/HelperMacros.h>
 
 class RequestTest:public CppUnit::TestFixture {
@@ -25,6 +27,10 @@ class RequestTest:public CppUnit::TestFixture {
   CPPUNIT_TEST(testSafeChar);
   CPPUNIT_TEST(testInnerLink);
   CPPUNIT_TEST(testMetalink);
+  CPPUNIT_TEST(testResolveHttpAuthConfig);
+  CPPUNIT_TEST(testResolveHttpAuthConfig_noCandidate);
+  CPPUNIT_TEST(testResolveHttpProxyAuthConfig);
+  CPPUNIT_TEST(testResolveFtpAuthConfig);
   CPPUNIT_TEST_SUITE_END();
   
 public:
@@ -48,6 +54,10 @@ public:
   void testSafeChar();
   void testInnerLink();
   void testMetalink();
+  void testResolveHttpAuthConfig();
+  void testResolveHttpAuthConfig_noCandidate();
+  void testResolveHttpProxyAuthConfig();
+  void testResolveFtpAuthConfig();
 };
 
 
@@ -293,3 +303,78 @@ void RequestTest::testMetalink() {
   bool v2 = req.setUrl("http://aria.rednoah.com/download/aria.tar.bz2#!metalink3!");
   CPPUNIT_ASSERT(!v2);
 }
+
+void RequestTest::testResolveHttpAuthConfig()
+{
+  Request req;
+  req.setUrl("http://localhost/download/aria2-1.0.0.tar.bz2");
+  // with DefaultAuthResolver
+  DefaultAuthResolverHandle defaultAuthResolver = new DefaultAuthResolver();
+  req.setHttpAuthResolver(defaultAuthResolver);
+  CPPUNIT_ASSERT(!req.resolveHttpAuthConfig().isNull());
+  CPPUNIT_ASSERT_EQUAL(string(":"),
+		       req.resolveHttpAuthConfig()->getAuthText());
+
+  // with Netrc
+  NetrcHandle netrc = new Netrc();
+  netrc->addAuthenticator(new DefaultAuthenticator("default", "defaultpassword", "defaultaccount"));
+  NetrcAuthResolverHandle netrcAuthResolver = new NetrcAuthResolver();
+  netrcAuthResolver->setNetrc(netrc);
+  req.setHttpAuthResolver(netrcAuthResolver);
+  AuthConfigHandle authConfig1 = req.resolveHttpAuthConfig();
+  CPPUNIT_ASSERT(!authConfig1.isNull());
+  CPPUNIT_ASSERT_EQUAL(string("default:defaultpassword"),
+		       authConfig1->getAuthText());
+
+  // with Netrc + user defined
+  AuthConfigHandle authConfig =
+    new AuthConfig("userDefinedUser", "userDefinedPassword");
+  netrcAuthResolver->setUserDefinedAuthConfig(authConfig);
+  AuthConfigHandle authConfig2 = req.resolveHttpAuthConfig();
+  CPPUNIT_ASSERT(!authConfig2.isNull());
+  CPPUNIT_ASSERT_EQUAL(string("userDefinedUser:userDefinedPassword"),
+		       authConfig2->getAuthText());
+}
+
+void RequestTest::testResolveHttpAuthConfig_noCandidate()
+{
+  Request req;
+  req.setUrl("http://localhost/download/aria2-1.0.0.tar.bz2");
+
+  DefaultAuthResolverHandle defaultAuthResolver = new DefaultAuthResolver();
+  req.setHttpAuthResolver(defaultAuthResolver);
+  CPPUNIT_ASSERT_EQUAL(string(":"),
+		       req.resolveHttpAuthConfig()->getAuthText());
+}
+
+void RequestTest::testResolveHttpProxyAuthConfig()
+{
+  Request req;
+  req.setUrl("http://localhost/download/aria2-1.0.0.tar.bz2");
+  // with Netrc
+  NetrcHandle netrc = new Netrc();
+  netrc->addAuthenticator(new DefaultAuthenticator("default", "defaultpassword", "defaultaccount"));
+  NetrcAuthResolverHandle netrcAuthResolver = new NetrcAuthResolver();
+  netrcAuthResolver->setNetrc(netrc);
+  req.setHttpProxyAuthResolver(netrcAuthResolver);
+  AuthConfigHandle authConfig1 = req.resolveHttpProxyAuthConfig();
+  CPPUNIT_ASSERT(!authConfig1.isNull());
+  CPPUNIT_ASSERT_EQUAL(string("default:defaultpassword"),
+		       authConfig1->getAuthText());
+}
+
+void RequestTest::testResolveFtpAuthConfig()
+{
+  Request req;
+  req.setUrl("http://localhost/download/aria2-1.0.0.tar.bz2");
+  // with Netrc
+  NetrcHandle netrc = new Netrc();
+  netrc->addAuthenticator(new DefaultAuthenticator("default", "defaultpassword", "defaultaccount"));
+  NetrcAuthResolverHandle netrcAuthResolver = new NetrcAuthResolver();
+  netrcAuthResolver->setNetrc(netrc);
+  req.setFtpAuthResolver(netrcAuthResolver);
+  AuthConfigHandle authConfig1 = req.resolveFtpAuthConfig();
+  CPPUNIT_ASSERT(!authConfig1.isNull());
+  CPPUNIT_ASSERT_EQUAL(string("default:defaultpassword"),
+		       authConfig1->getAuthText());
+}
